#
#   Pyrex - Code output module
#

import Naming
from Pyrex.Utils import open_new_file

class Generator:
  # f                file      output file
  # level            int       indentation level
  # bol              bool      beginning of line?
  # return_label     string    function return point label
  # error_label      string    error catch point label
  # continue_label   string    loop continue point label
  # break_label      string    loop break point label
  # label_counter    integer   counter for naming labels
  # in_try_finally   boolean   inside try of try...finally
  
  in_try_finally = 0
  
  def __init__(self, outfile_name):
    self.f = open_new_file(outfile_name)
    self.level = 0
    self.bol = 1
    self.label_counter = 1
    self.error_label = None
  
  def put(self, code):
    dl = code.count("{") - code.count("}")
    if dl < 0:
      self.level += dl
    if self.bol:
      self.indent()
    self.f.write(code)
    self.bol = 0
    if dl > 0:
      self.level += dl
  
  def putln(self, code = ""):
    if code:
      self.put(code)
    self.f.write("\n");
    self.bol = 1
  
  def increase_indent(self):
    self.level = self.level + 1
  
  def decrease_indent(self):
    self.level = self.level - 1
  
  def begin_block(self):
    self.putln("{")
    self.increase_indent()
  
  def end_block(self):
    self.decrease_indent()
    self.putln("}")
  
  def indent(self):
    self.f.write("  " * self.level)
  
  def mark_pos(self, pos):
    file, line, col = pos
    marker = '"%s":%s' % (file, line)
    #print ###
    #print marker ###
    self.putln("")
    self.putln("/* %s */" % marker)

  def init_labels(self):
    self.label_counter = 0
    self.return_label = self.new_label()
    self.new_error_label()
    self.continue_label = None
    self.break_label = None
  
  def new_label(self):
    n = self.label_counter
    self.label_counter = n + 1
    return "%s%d" % (Naming.label_prefix, n)
  
  def new_error_label(self):
    old_err_lbl = self.error_label
    self.error_label = self.new_label()
    return old_err_lbl
  
  def get_loop_labels(self):
    return (
      self.continue_label,
      self.break_label)
  
  def set_loop_labels(self, labels):
    (self.continue_label,
     self.break_label) = labels
  
  def new_loop_labels(self):
    old_labels = self.get_loop_labels()
    self.set_loop_labels(
      (self.new_label(), 
       self.new_label()))
    return old_labels
  
  def get_all_labels(self):
    return (
      self.continue_label,
      self.break_label,
      self.return_label,
      self.error_label)

  def set_all_labels(self, labels):
    (self.continue_label,
     self.break_label,
     self.return_label,
     self.error_label) = labels

  def all_new_labels(self):
    old_labels = self.get_all_labels()
    new_labels = []
    for old_label in old_labels:
      if old_label:
        new_labels.append(self.new_label())
      else:
        new_labels.append(old_label)
    self.set_all_labels(new_labels)
    return old_labels

  def put_label(self, lbl):
    self.putln("%s:;" % lbl)
  
  def put_var_declarations(self, entries, static = 0, dll_linkage = None):
    for entry in entries:
      if not entry.in_cinclude:
        self.put_var_declaration(entry, static, dll_linkage)
  
  def put_var_declaration(self, entry, static = 0, dll_linkage = None):
    if entry.extern_flag:
      self.put("extern ")
    elif static and not entry.public_flag:
      self.put("static ")
    self.put(entry.type.declaration_code(entry.cname,
      dll_linkage = entry.public_flag and dll_linkage))
    if entry.init is not None:
      self.put(" = %s" % entry.type.literal_code(entry.init))
    self.putln(";")
  
  def put_incref(self, cname):
    self.putln("Py_INCREF(%s);" % cname)
  
  def put_decref(self, cname):
    self.putln("Py_DECREF(%s);" % cname)
  
  def put_decref_clear(self, cname):
    self.putln("Py_DECREF(%s); %s = 0;" % (cname, cname))
  
  def put_xdecref(self, cname):
    self.putln("Py_XDECREF(%s);" % cname)
  
  def put_xdecref_clear(self, cname):
    self.putln("Py_XDECREF(%s); %s = 0;" % (cname, cname))

  def put_var_decref(self, entry):
    if entry.type.is_pyobject:
      self.put_decref(entry.cname)
  
  def put_var_xdecref(self, entry):
    if entry.type.is_pyobject:
      self.put_xdecref(entry.cname)
  
  def put_var_xdecref_clear(self, entry):
    if entry.type.is_pyobject:
      self.put_xdecref_clear(entry.cname)
  
  def put_var_decrefs(self, entries):
    for entry in entries:
      if entry.xdecref_cleanup:
        self.put_var_xdecref(entry)
      else:
        self.put_var_decref(entry)
  
  def put_var_xdecrefs(self, entries):
    for entry in entries:
      self.put_var_xdecref(entry)
  
  def put_var_xdecrefs_clear(self, entries):
    for entry in entries:
      self.put_var_xdecref_clear(entry)
  
  def put_init_to_py_none(self, cname):
    self.putln("%s = Py_None; Py_INCREF(%s);" % (cname, cname))
  
  def put_init_var_to_py_none(self, entry, template = "%s"):
    code = template % entry.cname
    if entry.type.is_extension_type:
      code = "((PyObject*)%s)" % code
    self.put_init_to_py_none(code)

  def put_pymethoddef(self, entry, term):
    if entry.doc:
      doc_code = entry.doc_cname
    else:
      doc_code = 0
    self.putln(
      '{"%s", (PyCFunction)%s, METH_VARARGS|METH_KEYWORDS, %s}%s' % (
        entry.name, 
        entry.func_cname, 
        doc_code,
        term))
  
  def error_goto(self, pos):
    return "{%s = %s; goto %s;}" % (
      Naming.lineno_cname,
      pos[1],
      self.error_label)

