#
#   Pyrex - Parse tree nodes for expressions
#

from string import join

from Errors import error, InternalError
import Naming
from Nodes import Node
import PyrexTypes

from DebugFlags import debug_disposal_code

class ExprNode(Node):
  #  type                     PyrexType   Type of the result
  #  result                   string      C code fragment
  #  is_temp                  boolean     Result is in a temporary local
  #  is_sequence_constructor  boolean     Is a list or tuple constructor expression
  #
  #  Special processing phases for ExprNodes:
  #
  #    analyse_target_declaration
  #      Called during the analyse_declarations phase to analyse
  #      the LHS of an assignment or argument of a del statement.
  #
  #    analyse_target_expression
  #      Called during the analyse_expressions phase to analyse
  #      the LHS of an assignment or argument of a del statement.
  #
  #    check_const
  #      Tests whether this is a legal constant expression, and
  #      reports an error otherwise.
  #
  #    check_addr_const
  #      Tests whether this is an lvalue with a constant address,
  #      and reports an error otherwise.
  #
  #    analyse_expressions
  #      - Call analyse_expressions on all sub-expressions.
  #      - Check operand types, set type and result of this node.
  #      - Call release_subexpr_temps on this node.
  #
  #    release_subexpr_temps
  #      Perform release_temp on all sub-expressions.
  #
  #    generate_evaluation_code
  #      Called during the generate_code phase.
  #      - Call generate_evaluation_code for sub-expressions.
  #      - Generate code to evaluate the result, if needed.
  #      - If result is temporary, call generate_subexpr_disposal_code.
  #
  #    generate_assignment_code
  #      Called on the LHS of an assignment during the 
  #      generate_code phase.
  #      - Call generate_evaluation_code for sub-expressions.
  #      - Generate code to perform the assignment.
  #      - If the assignment absorbed a reference, call
  #        generate_post_assignment_code on the RHS,
  #        otherwise call generate_disposal_code on it.
  #
  #    generate_deletion_code
  #      Called on an argument of a del statement during the 
  #      generate_code phase.
  #      - Call generate_evaluation_code for sub-expressions.
  #      - Generate code to perform the deletion.
  #      - Call generate_disposal_code on all sub-expressions.
  #
  #    generate_subexpr_disposal_code
  #      Call generate_disposal_code for all sub-expressions.
  #      This will only be called if the result is not in
  #      a temporary. Therefore, nodes for which the result
  #      is always in a temporary need not implement it.
  #
  
  is_temp = 0
  is_sequence_constructor = 0
  
  def analyse_target_declaration(self, env):
    error(self.pos, "Cannot assign to or delete this")
  
  def analyse_expressions(self, env):
    raise InternalError(
      "%s.analyse_expressions not implemented" %
        self.__class__.__name__)		
  
  def analyse_target_expression(self, env):
    # If this node is not legal as the LHS of an assignment,
    # an error will have been reported during declaration 
    # analysis phase.
    self.analyse_expressions(env)
  
  def analyse_const_expression(self, env):
    #  Called during the analyse_declarations phase of a
    #  constant expression. Analyses the expression's type,
    #  checks whether it is a legal const expression,
    #  and determines its value.
    self.analyse_expressions(env)
    self.check_const()
  
  def check_const(self):
    self.not_const()
  
  def not_const(self):
    error(self.pos, "Not allowed in a constant expression")
  
  def check_addr_const(self):
    self.addr_not_const()
  
  def addr_not_const(self):
    error(self.pos, "Address is not constant")
  
  def analyse_boolean_expression(self, env):
    #  Analyse expression and coerce to a boolean.
    self.analyse_expressions(env)
    return self.coerce_to_boolean(env)
  
  def analyse_temp_boolean_expression(self, env):
    # Analyse boolean expression and coerce result into
    # a temporary. This is used when a branch is to be
    # performed on the result and we won't have an
    # opportunity to ensure disposal code is executed
    # afterwards. By forcing the result into a temporary,
    # we ensure that all disposal has been done by the
    # time we get the result.
    bool = self.analyse_boolean_expression(env)
    return bool.coerce_to_temp(env)
  
  def coerce_to(self, dst_type, env):
    #   Coerce the result so that it can be assigned to
    #   something of type dst_type. If processing is necessary,
    #   wraps this node in a coercion node and returns that.
    #   Otherwise, returns this node unchanged.
    #
    #   This method is called during the analyse_expressions
    #   phase of the src_node's processing.
    src_type = self.type
    src_is_py_type = src_type.is_pyobject
    dst_is_py_type = dst_type.is_pyobject
    if dst_is_py_type:
      result = self
      if not src_is_py_type:
        result = CoerceToPyTypeNode(result, env)
      # src and dst are now both py types
      if not dst_type.same_as(result.type):
        if dst_type.is_extension_type:
          result = PyTypeTestNode(result, dst_type, env)
        #elif src_type.is_extension_type:
        #	result = CastToPyObjectNode(result, env)
      return result
    elif src_is_py_type:
      return CoerceFromPyTypeNode(dst_type, self, env)
    else: # neither src nor dst are py types
      if src_type.is_string and dst_type.is_int:
        return CoerceToCharNode(self)
      if not dst_type.assignable_from(src_type):
        error(self.pos, "Cannot assign type '%s' to '%s'" %
          (src_type, dst_type))
      return self
  
  def coerce_to_pyobject(self, env):
    return self.coerce_to(PyrexTypes.py_object_type, env)

  def coerce_to_boolean(self, env):
    #  Coerce result to something acceptable as
    #  a boolean value.
    type = self.type
    if type.is_pyobject or type.is_ptr or type.is_float:
      return CoerceToBooleanNode(self, env)
    else:
      if not type.is_int:
        error(operand.pos, 
          "Type '%s' not acceptable as a boolean" % type)
      return self
  
  def coerce_to_integer(self, env):
    # If not already some C integer type, coerce to longint.
    if self.type.is_int:
      return self
    else:
      return self.coerce_to(PyrexTypes.c_long_type, env)
  
  def coerce_to_temp(self, env):
    #  Ensure that result is in a temporary.
    if self.is_temp:
      return self
    else:
      return CoerceToTempNode(self, env)
      
  def allocate_temp(self, env):
    #  Allocate a temporary of the appropriate type to hold
    #  the result, and mark node as temporary.
    if not self.type.is_void:
      self.result = env.allocate_temp(self.type)
    else:
      self.result = None
    self.is_temp = 1

  def release_temp(self, env):
    #  If result is in a temporary local, release the temporary for re-use.
    #  Otherwise, apply recursively to sub-nodes.
    #  Called by parent node at the end of its analyse_expressions phase.
    if self.is_temp:
      env.release_temp(self.result)
    else:
      self.release_subexpr_temps(env)
  
  def release_subexpr_temps(self, env):
    raise InternalError(
      "%s.release_subexpr_temps not implemented" %
        self.__class__.__name__)
  
  def make_owned_reference(self, code):
    #  If result is a pyobject, make sure we own
    #  a reference to it.
    if self.type.is_pyobject and not self.is_temp:
      code.put_incref(self.result)
  
  def generate_evaluation_code(self, code):
    raise InternalError(
      "%s.generate_evaluation_code not implemented" %
        self.__class__.__name__)
  
  def generate_assignment_code(self, rhs, code):
    # If this node is not legal as the LHS of an assignment,
    # an error will have been reported earlier.
    pass
  
  def generate_deletion_code(self, code):
    # If this node is not legal as a del argument,
    # an error will have been reported earlier.
    pass
  
  def generate_disposal_code(self, code):
    # If necessary, generate code to dispose of 
    # temporary Python reference.
    if debug_disposal_code:
      print "ExprNode.generate_disposal_code:"
      print "...generating disposal code for", self
    if self.is_temp:
      if self.type.is_pyobject:
        code.put_decref_clear(self.result)
    else:
      if debug_disposal_code:
        print "ExprNode.generate_disposal_code:"
        print "...generating subexpr disposal code for", self
      self.generate_subexpr_disposal_code(code)
  
  def generate_subexpr_disposal_code(self, code):
    raise InternalError(
      "%s.generate_subexpr_disposal_code not implemented" %
        self.__class__.__name__)
  
  def generate_post_assignment_code(self, code):
    # Same as generate_disposal_code except that
    # assignment will have absorbed a reference to
    # the result if it is a Python object.
    if self.is_temp:
      if self.type.is_pyobject:
        code.putln("%s = 0;" % self.result)
    else:
      if debug_disposal_code:
        print "ExprNode.generate_post_assignment_code:"
        print "...generating subexpression disposal code for", self
      self.generate_subexpr_disposal_code(code)
  
  def is_lvalue(self):
    return 0
  
  def is_ephemeral(self):
    #  An ephemeral node is one whose result is in
    #  a Python temporary and we know there are no
    #  other references to it. Certain operations are
    #  disallowed on such values, since they are
    #  bound to result in a dangling pointer.
    return self.type.is_pyobject and self.is_temp


class AtomicExprNode(ExprNode):
  #  Base class for expression nodes which have
  #  no sub-expressions.
  
  def release_subexpr_temps(self, env):
    pass
  
  def generate_subexpr_disposal_code(self, code):
    pass


class NoneNode(AtomicExprNode):
  #  The constant value None
  
  def analyse_expressions(self, env):
    self.type = PyrexTypes.py_object_type
    self.result = "Py_None"

  def generate_evaluation_code(self, code):
    pass


class EllipsisNode(AtomicExprNode):
  #  '...' in a subscript list.
  
  def analyse_expressions(self, env):
    self.type = PyrexTypes.py_object_type
    self.result = "Py_Ellipsis"
  
  def generate_evaluation_code(self, code):
    pass


class ConstNode(AtomicExprNode):
  # Abstract base type for literal constant nodes.
  #
  # value     string      C code fragment
  
  is_literal = 1
  
  def analyse_expressions(self, env):
    self.result = str(self.value)
  
  def check_const(self):
    pass

  def generate_evaluation_code(self, code):
    pass

class NullNode(ConstNode):
  type = PyrexTypes.c_null_ptr_type
  value = "0"

class IntNode(ConstNode):
  type = PyrexTypes.c_long_type

class FloatNode(ConstNode):
  type = PyrexTypes.c_double_type

class StringNode(ConstNode):
  type = PyrexTypes.c_char_ptr_type
  
  def analyse_expressions(self, env):
    entry = env.add_string_const(self.value)
    self.result = entry.cname

class ImagNode(ExprNode):
  #  Imaginary number literal
  #
  #  value   float    imaginary part
  
  def analyse_expressions(self, env):
    self.type = PyrexTypes.py_object_type
    self.allocate_temp(env)
  
  def generate_evaluation_code(self, code):
    code.putln(
      "%s = PyComplex_FromDoubles(0.0, %s); if (!%s) %s" % (
        self.result,
        self.value,
        self.result,
        code.error_goto(self.pos)))


class NameNode(AtomicExprNode):
  #  Reference to a local or global variable name.
  #
  #  name    string    Python name of the variable
  #  entry   Entry     Symbol table entry
  
  is_name = 1
  
  def analyse_target_declaration(self, env):
    #print "NameNode.analyse_target_declaration:", self.name ###
    self.entry = env.lookup_here(self.name)
    if not self.entry:
      self.entry = env.declare_var(self.name, 
        PyrexTypes.py_object_type, self.pos)
    self.check_identifier_kind()
  
  def check_identifier_kind(self):
    entry = self.entry
    if not (entry.is_const or entry.is_variable 
      or entry.is_builtin or entry.is_cfunction):
        if self.entry.as_variable:
          self.entry = self.entry.as_variable
        else:
          error(self.pos, 
            "'%s' is not a constant, variable or function identifier" % self.name)
  
  def analyse_expressions(self, env):
    #print "NameNode.analyse_expressions:", self.name ###
    self.entry = env.lookup(self.name)
    if not self.entry:
      self.entry = env.declare_builtin(self.name, self.pos)
    self.check_identifier_kind()
    self.type = self.entry.type
    # Reference to C array turns into pointer to first element.
    if self.type.is_array:
      self.type = self.type.element_ptr_type()
    if self.entry.is_pyglobal or self.entry.is_builtin:
      if not self.type.is_pyobject:
        raise InternalError("Python global or builtin not a Python object")
      self.allocate_temp(env)
      env.use_utility_code(get_name_utility_code)
    else:
      self.cvar_result()
  
  def cvar_result(self):
    self.result = self.entry.cname
    if self.type.is_extension_type and not self.entry.is_self_arg:
      self.result = "((PyObject *)%s)" % self.result
  
  def analyse_target_expression(self, env):
    #print "NameNode.analyse_target_expression:", self.name ###
    # Entry now remembered in analyse_target_declaration
    #self.entry = env.lookup_here(self.name)
    #if not self.entry:
    #	# Shouldn't happen, because of implicit declarations
    #	raise InternalError("Assignment target not found during analyse_expressions phase")
    if self.is_lvalue():
      self.type = self.entry.type
    else:
      error(self.pos, "Assignment to non-lvalue '%s'"
        % self.name)
      #self.entry = None
      self.type = PyrexTypes.error_type
    self.cvar_result()
  
  def check_const(self):
    entry = self.entry
    if not (entry.is_const or entry.is_cfunction):
      self.not_const()
  
  def check_addr_const(self):
    entry = self.entry
    if not (entry.is_cglobal or entry.is_cfunction):
      self.addr_not_const()

  def is_lvalue(self):
    return self.entry.is_variable and \
      not self.entry.type.is_array and \
      not self.entry.is_readonly
  
  def is_ephemeral(self):
    #  Name nodes are never ephemeral, even if the
    #  result is in a temporary.
    return 0

  def generate_evaluation_code(self, code):
    #print "NameNode.generate_evaluation_code:", self.name ###
    entry = self.entry
    if entry is None:
      return # There was an error earlier
    if entry.is_pyglobal or entry.is_builtin:
      if entry.is_builtin:
        namespace = Naming.builtins_cname
      else: # entry.is_pyglobal
        namespace = entry.namespace_cname
      code.putln(
        '%s = __Pyx_GetName(%s, "%s"); if (!%s) %s' % (
        self.result,
        namespace, 
        self.entry.name,
        self.result, 
        code.error_goto(self.pos)))		
  
  def generate_assignment_code(self, rhs, code):
    #print "NameNode.generate_assignment_code:", self.name ###
    if self.entry is None:
      return # There was an error earlier
    if self.entry.is_pyglobal:
      namespace = self.entry.namespace_cname
      code.putln(
        'if (PyObject_SetAttrString(%s, "%s", %s) < 0) %s' % (
          namespace, 
          self.entry.name, 
          rhs.result, 
          code.error_goto(self.pos)))
      #if rhs.is_temp:
      #	code.put_decref(rhs.result)
      if debug_disposal_code:
        print "NameNode.generate_assignment_code:"
        print "...generating disposal code for", rhs
      rhs.generate_disposal_code(code)
    else:
      if self.type.is_pyobject:
        #if not rhs.is_temp:
        #	code.put_incref(rhs.result)
        rhs.make_owned_reference(code)
        code.put_decref(self.result)
      code.putln('%s = %s;' % (self.result, rhs.result))
      if debug_disposal_code:
        print "NameNode.generate_assignment_code:"
        print "...generating post-assignment code for", rhs
      rhs.generate_post_assignment_code(code)
  
  def generate_deletion_code(self, code):
    if self.entry is None:
      return # There was an error earlier
    if not self.entry.is_pyglobal:
      error(self.pos, "Deletion of local name not supported")
      return
    code.putln(
      'if (PyObject_DelAttrString(%s, "%s") < 0) %s' % (
        Naming.module_cname,
        self.entry.name,
        code.error_goto(self.pos)))
      

class BackquoteNode(ExprNode):
  #  `expr`
  #
  #  arg    ExprNode
  
  def analyse_expressions(self, env):
    self.arg.analyse_expressions(env)
    self.arg = self.arg.coerce_to_pyobject(env)
    self.type = PyrexTypes.py_object_type
    self.allocate_temp(env)
    self.arg.release_temp(env)
  
  def generate_evaluation_code(self, code):
    self.arg.generate_evaluation_code(code)
    code.putln(
      "%s = PyObject_Repr(%s); if (!%s) %s" % (
        self.result,
        self.arg.result,
        self.result,
        code.error_goto(self.pos)))
    self.arg.generate_disposal_code(code)

  # generate_subexpr_disposal_code never called because
  # result is always temporary


class ImportNode(ExprNode):
  #  Used as part of import statement implementation.
  #  Implements result = 
  #    __import__(module_name, globals(), None, name_list)
  #
  #  module_name   StringNode         dotted name of module
  #  name_list     ListNode or None   list of names to be imported

  def analyse_expressions(self, env):
    self.module_name.analyse_expressions(env)
    self.module_name = self.module_name.coerce_to_pyobject(env)
    if self.name_list:
      self.name_list.analyse_expressions(env)
    self.type = PyrexTypes.py_object_type
    self.allocate_temp(env)
    self.module_name.release_temp(env)
    if self.name_list:
      self.name_list.release_temp(env)
    env.use_utility_code(import_utility_code)
  
  def generate_evaluation_code(self, code):
    self.module_name.generate_evaluation_code(code)
    if self.name_list:
      self.name_list.generate_evaluation_code(code)
      name_list_code = self.name_list.result
    else:
      name_list_code = "0"
    code.putln(
      "%s = __Pyx_Import(%s, %s); if (!%s) %s" % (
        self.result,
        self.module_name.result,
        name_list_code,
        self.result,
        code.error_goto(self.pos)))
    self.module_name.generate_disposal_code(code)
    if self.name_list:
      self.name_list.generate_disposal_code(code)


class IteratorNode(ExprNode):
  #  Used as part of for statement implementation.
  #  Implements result = iter(sequence)
  #
  #  sequence   ExprNode
  
  def analyse_expressions(self, env):
    self.sequence.analyse_expressions(env)
    self.sequence = self.sequence.coerce_to_pyobject(env)
    self.type = PyrexTypes.py_object_type
    self.allocate_temp(env)
    self.sequence.release_temp(env)
  
  def generate_evaluation_code(self, code):
    self.sequence.generate_evaluation_code(code)
    code.putln(
      "%s = PyObject_GetIter(%s); if (!%s) %s" % (
        self.result,
        self.sequence.result,
        self.result,
        code.error_goto(self.pos)))
    self.sequence.generate_disposal_code(code)


class NextNode(ExprNode):
  #  Used as part of for statement implementation.
  #  Implements result = iterator.next()
  #  Created during analyse_expressions phase.
  #
  #  iterator   ExprNode
  
  def __init__(self, iterator, env):
    self.pos = iterator.pos
    self.iterator = iterator
    self.type = PyrexTypes.py_object_type
    self.allocate_temp(env)
  
  def generate_evaluation_code(self, code):
    code.putln(
      "%s = PyIter_Next(%s);" % (
        self.result,
        self.iterator.result))
    code.putln(
      "if (!%s) {" %
        self.result)
    code.putln(
        "if (PyErr_Occurred()) %s" %
          code.error_goto(self.pos))
    code.putln(
        "break;")
    code.putln(
      "}")
    # Don't generate disposal code for iterator, because
    # we don't own it.
    

class ExcValueNode(ExprNode):
  #  Node created during analyse_expressions phase
  #  of an ExceptClauseNode to fetch the current
  #  exception value.
  
  def __init__(self, pos, env):
    ExprNode.__init__(self, pos)
    self.type = PyrexTypes.py_object_type
    self.allocate_temp(env)
    env.use_utility_code(get_exception_utility_code)
  
  def generate_evaluation_code(self, code):
    code.putln(
      "%s = __Pyx_GetExcValue(); if (!%s) %s" % (
        self.result,
        self.result,
        code.error_goto(self.pos)))


class TempNode(ExprNode):
  #  Node created during analyse_expressions phase
  #  of some nodes to hold a temporary value.
  
  def __init__(self, pos, type, env):
    ExprNode.__init__(self, pos)
    self.type = type
    self.allocate_temp(env)
  
  def generate_evaluation_code(self, code):
    pass


class PyTempNode(TempNode):
  #  TempNode holding a Python value.
  
  def __init__(self, pos, env):
    TempNode.__init__(self, pos, PyrexTypes.py_object_type, env)


#-------------------------------------------------------------------
#
#  Trailer nodes
#
#-------------------------------------------------------------------

class IndexNode(ExprNode):
  #  Sequence indexing.
  #
  #  base     ExprNode
  #  index    ExprNode
  
  def analyse_target_declaration(self, env):
    pass
  
  def analyse_expressions(self, env):
    self.base.analyse_expressions(env)
    self.index.analyse_expressions(env)
    if self.base.type.is_pyobject: 
      self.index = self.index.coerce_to_pyobject(env)
      self.type = PyrexTypes.py_object_type
      self.allocate_temp(env)
    else:
      if self.base.type.is_ptr or self.base.type.is_array:
        self.type = self.base.type.base_type
      else:
        error(self.pos,
          "Attempting to index non-array type '%s'" %
            self.base.type)
        self.type = PyrexTypes.error_type
      if self.index.type.is_pyobject:
        self.index = self.index.coerce_to(
          PyrexTypes.c_int_type, env)
      if not self.index.type.is_int:
        error(self.pos,
          "Invalid index type '%s'" %
            self.index.type)
      self.result = "(%s[%s])" % (
        self.base.result, self.index.result)
    self.release_subexpr_temps(env)
  
  def check_addr_const(self):
    #print "IndexNode.check_addr_const" ###
    self.base.check_addr_const()
    self.index.check_const()
  
  def is_lvalue(self):
    return 1
  
  def release_subexpr_temps(self, env):
    self.base.release_temp(env)
    self.index.release_temp(env)
  
  def generate_evaluation_code(self, code):
    self.generate_subexpr_evaluation_code(code)
    if self.type.is_pyobject:
      code.putln(
        "%s = PyObject_GetItem(%s, %s); if (!%s) %s" % (
          self.result,
          self.base.result,
          self.index.result,
          self.result,
          code.error_goto(self.pos)))
      if debug_disposal_code:
        print "IndexNode.generate_evaluation_code:"
        print "...generating subexpr disposal code for", self
      self.generate_subexpr_disposal_code(code)
  
  def generate_assignment_code(self, rhs, code):
    self.generate_subexpr_evaluation_code(code)
    if self.type.is_pyobject:
      code.putln(
        "if (PyObject_SetItem(%s, %s, %s) < 0) %s" % (
          self.base.result,
          self.index.result,
          rhs.result,
          code.error_goto(self.pos)))
      if debug_disposal_code:
        print "IndexNode.generate_assignment_code:"
        print "...generating subexpr disposal code for", self
      self.generate_subexpr_disposal_code(code)
    else:
      code.putln(
        "%s = %s;" % (
          self.result, rhs.result))
    if debug_disposal_code:
      print "IndexNode.generate_assignment_code:"
      print "...generating disposal code for", rhs
    rhs.generate_disposal_code(code)
  
  def generate_deletion_code(self, code):
    self.generate_subexpr_evaluation_code(code)
    code.putln(
      "if (PyObject_DelItem(%s, %s) < 0) %s" % (
        self.base.result,
        self.index.result,
        code.error_goto(self.pos)))
    self.generate_subexpr_disposal_code(code)
  
  def generate_subexpr_evaluation_code(self, code):
    self.base.generate_evaluation_code(code)
    self.index.generate_evaluation_code(code)
          
  def generate_subexpr_disposal_code(self, code):
    if debug_disposal_code:
      print "IndexNode.generate_subexpr_disposal_code:"
      print "...generating disposal code for", self.base
      print "...and", self.index
    self.base.generate_disposal_code(code)
    self.index.generate_disposal_code(code)


class SliceIndexNode(ExprNode):
  #  2-element slice indexing
  #
  #  base      ExprNode
  #  start     ExprNode or None
  #  stop      ExprNode or None
  
  def analyse_expressions(self, env):
    self.base.analyse_expressions(env)
    if self.start:
      self.start.analyse_expressions(env)
    if self.stop:
      self.stop.analyse_expressions(env)
    self.base = self.base.coerce_to_pyobject(env)
    c_int = PyrexTypes.c_int_type
    if self.start:
      self.start = self.start.coerce_to(c_int, env)
    if self.stop:
      self.stop = self.stop.coerce_to(c_int, env)
    self.type = PyrexTypes.py_object_type
    self.allocate_temp(env)
    self.base.release_temp(env)
    if self.start:
      self.start.release_temp(env)
    if self.stop:
      self.stop.release_temp(env)
  
  def generate_evaluation_code(self, code):
    self.generate_subexpr_evaluation_code(code)
    code.putln(
      "%s = PySequence_GetSlice(%s, %s, %s); if (!%s) %s" % (
        self.result,
        self.base.result,
        self.start_code(),
        self.stop_code(),
        self.result,
        code.error_goto(self.pos)))
    self.generate_subexpr_disposal_code(code)
  
  def generate_assignment_code(self, rhs, code):
    self.generate_subexpr_evaluation_code(code)
    code.putln(
      "if (PySequence_SetSlice(%s, %s, %s, %s) < 0) %s" (
        self.base.result,
        self.start_code(),
        self.stop_code(),
        rhs.result,
        code.error_goto(self.pos)))
    self.generate_subexpr_disposal_code(code)
    rhs.generate_disposal_code(code)

  def generate_deletion_code(self, code):
    self.generate_subexpr_evaluation_code(code)
    code.putln(
      "if (PySequence_DelSlice(%s, %s, %s) < 0) %s" (
        self.base.result,
        self.start_code(),
        self.stop_code(),
        code.error_goto(self.pos)))
    self.generate_subexpr_disposal_code(code)
  
  def start_code(self):
    if self.start:
      return self.start.result
    else:
      return "0"
  
  def stop_code(self):
    if self.stop:
      return self.stop.result
    else:
      return "0x7fffffff"

  def generate_subexpr_evaluation_code(self, code):
    self.base.generate_evaluation_code(code)
    if self.start:
      self.start.generate_evaluation_code(code)
    if self.stop:
      self.stop.generate_evaluation_code(code)
  
  def generate_subexpr_disposal_code(self, code):
    self.base.generate_disposal_code(code)
    if self.start:
      self.start.generate_disposal_code(code)
    if self.stop:
      self.stop.generate_disposal_code(code)
  

class SliceNode(ExprNode):
  #  start:stop:step in subscript list
  #
  #  start     ExprNode
  #  stop      ExprNode
  #  step      ExprNode
  
  def analyse_expressions(self, env):
    self.start.analyse_expressions(env)
    self.stop.analyse_expressions(env)
    self.step.analyse_expressions(env)
    self.start = self.start.coerce_to_pyobject(env)
    self.stop = self.stop.coerce_to_pyobject(env)
    self.step = self.step.coerce_to_pyobject(env)
    self.type = PyrexTypes.py_object_type
    self.allocate_temp(env)
    self.release_subexpr_temps(env)
  
  def release_subexpr_temps(self, env):
    self.start.release_temp(env)
    self.stop.release_temp(env)
    self.step.release_temp(env)

  def generate_evaluation_code(self, code):
    self.start.generate_evaluation_code(code)
    self.stop.generate_evaluation_code(code)
    self.step.generate_evaluation_code(code)
    code.putln(
      "%s = PySlice_New(%s, %s, %s); if (!%s) %s" % (
        self.result,
        self.start.result, 
        self.stop.result, 
        self.step.result,
        self.result,
        code.error_goto(self.pos)))
    if debug_disposal_code:
      print "SliceNode.generate_evaluation_code:"
      print "...generating subexpr disposal code for", self
    self.generate_subexpr_disposal_code(code)
  
  def generate_subexpr_disposal_code(self, code):
    if debug_disposal_code:
      print "SliceNode.generate_subexpr_disposal_code:"
      print "...generating disposal code for", self.start
      print "...and", self.stop
      print "...and", self.step
    self.start.generate_disposal_code(code)
    self.stop.generate_disposal_code(code)
    self.step.generate_disposal_code(code)


class SimpleCallNode(ExprNode):
  #  Function call without keyword, * or ** args.
  #
  #  function     ExprNode
  #  args         [ExprNode]
  #  arg_tuple    ExprNode     used internally
  
  arg_tuple = None
  
  def analyse_expressions(self, env):
    self.function.analyse_expressions(env)
    if self.function.type.is_pyobject:
      self.arg_tuple = TupleNode(self.pos, args = self.args)
      self.args = None
      self.arg_tuple.analyse_expressions(env)
      self.type = PyrexTypes.py_object_type
      self.allocate_temp(env)
      self.arg_tuple.release_temp(env)
    else:
      for arg in self.args:
        arg.analyse_expressions(env)
      self.analyse_c_function_call(env)
      for arg in self.args:
        arg.release_temp(env)
    self.function.release_temp(env)
  
  def analyse_c_function_call(self, env):
    func_type = self.function.type
    # Coerce function pointer to function
    if func_type.is_ptr:
      func_type = func_type.base_type
      self.function.type = func_type
    # Check function type
    if not func_type.is_cfunction:
      if not func_type.is_error:
        error(self.pos, "Calling non-function type '%s'" %
          func_type)
      self.type = PyrexTypes.error_type
      self.result = "<error>"
      return
    # Check no. of args
    expected_nargs = len(func_type.args)
    actual_nargs = len(self.args)
    if actual_nargs < expected_nargs \
      or (not func_type.has_varargs and actual_nargs > expected_nargs):
        expected_str = str(expected_nargs)
        if func_type.has_varargs:
          expected_str = "at least " + expected_str
        error(self.pos, 
          "Call with wrong number of arguments (expected %s, got %s)"
            % (expected_str, actual_nargs))
        self.type = PyrexTypes.error_type
        self.result = "<error>"
        return
    # Coerce arguments
    for i in range(expected_nargs):
      formal_type = func_type.args[i].type
      self.args[i] = self.args[i].coerce_to(formal_type, env)
    for i in range(expected_nargs, actual_nargs):
      if self.args[i].type.is_pyobject:
        error(self.args[i].pos, 
          "Python object cannot be passed as a varargs parameter")
    # Calc result type and code fragment
    self.type = func_type.return_type
    if self.type.is_pyobject \
      or func_type.exception_value is not None \
      or func_type.exception_check:
        self.allocate_temp(env)
    else:
      self.result = self.c_call_code()
  
  def c_call_code(self):
    arg_list_code = []
    for arg in self.args:
      arg_code = arg.result
      if arg.type.is_extension_type:
        arg_code = "((%s)%s)" % (arg.type.declaration_code(""),
          arg_code)
      arg_list_code.append(arg_code)
    return "%s(%s)" % (self.function.result,
      join(arg_list_code, ","))
  
  def release_subexpr_temps(self, env):
    self.function.release_temp(env)
    if self.args:
      for arg in self.args:
        arg.release_temp(env)
      if self.arg_tuple:
        self.arg_tuple.release_temp(env)

  def generate_evaluation_code(self, code):
    self.function.generate_evaluation_code(code)
    if self.function.type.is_pyobject:
      self.arg_tuple.generate_evaluation_code(code)
      code.putln(
        "%s = PyObject_CallObject(%s, %s); if (!%s) %s" % (
          self.result,
          self.function.result,
          self.arg_tuple.result,
          self.result,
          code.error_goto(self.pos)))
    else:
      for arg in self.args:
        arg.generate_evaluation_code(code)
      exc_checks = []
      if self.type.is_pyobject:
        exc_checks.append("!%s" % self.result)
      else:
        exc_val = self.function.type.exception_value
        exc_check = self.function.type.exception_check
        if exc_val is not None:
          exc_checks.append("%s == %s" % (self.result, exc_val))
        if exc_check:
          exc_checks.append("PyErr_Occurred()")
      if self.is_temp or exc_checks:
        if self.result:
          lhs = "%s = " % self.result
        else:
          lhs = ""
        code.putln(
          "%s%s; if (%s) %s" % (
            lhs,
            self.c_call_code(),
            " && ".join(exc_checks),
            code.error_goto(self.pos)))
    if self.is_temp:
      self.generate_subexpr_disposal_code(code)

  def generate_subexpr_disposal_code(self, code):
    self.function.generate_disposal_code(code)
    if self.arg_tuple:
      self.arg_tuple.generate_disposal_code(code)
    if self.args:
      for arg in self.args:
        arg.generate_disposal_code(code)
  

class GeneralCallNode(ExprNode):
  #  General Python function call, including keyword,
  #  * and ** arguments.
  #
  #  function         ExprNode
  #  positional_args  ExprNode          Tuple of positional arguments
  #  keyword_args     ExprNode or None  Dict of keyword arguments
  #  starstar_arg     ExprNode or None  Dict of extra keyword args
  def analyse_expressions(self, env):
    self.function.analyse_expressions(env)
    self.positional_args.analyse_expressions(env)
    if self.keyword_args:
      self.keyword_args.analyse_expressions(env)
    if self.starstar_arg:
      self.starstar_arg.analyse_expressions(env)
    self.function = self.function.coerce_to_pyobject(env)
    self.positional_args = \
      self.positional_args.coerce_to_pyobject(env)
    #if self.keyword_args:
    #	self.keyword_args = \
    #		self.keyword_args.coerce_to_pyobject(env)
    if self.starstar_arg:
      self.starstar_arg = \
        self.starstar_arg.coerce_to_pyobject(env)
    self.type = PyrexTypes.py_object_type
    self.allocate_temp(env)
    self.release_subexpr_temps(env)
  
  def release_subexpr_temps(self, env):
    self.function.release_temp(env)
    self.positional_args.release_temp(env)
    if self.keyword_args:
      self.keyword_args.release_temp(env)
    if self.starstar_arg:
      self.starstar_arg.release_temp(env)
  
  def generate_evaluation_code(self, code):
    self.function.generate_evaluation_code(code)
    self.positional_args.generate_evaluation_code(code)
    if self.keyword_args:
      self.keyword_args.generate_evaluation_code(code)
    if self.keyword_args and self.starstar_arg:
      code.putln(
        "if (PyDict_Update(%s, %s) < 0) %s" % (
          self.keyword_args.result, 
          self.starstar_arg.result,
          code.error_goto(self.pos)))
      keyword_code = self.keyword_args.result
    elif self.keyword_args:
      keyword_code = self.keyword_args.result
    elif self.starstar_arg:
      keyword_code = self.starstar_arg.result
    else:
      keyword_code = None
    if not keyword_code:
      call_code = "PyObject_CallObject(%s, %s)" % (
        self.function.result,
        self.positional_args.result)
    else:
      call_code = "PyEval_CallObjectWithKeywords(%s, %s, %s)" % (
        self.function.result,
        self.positional_args.result,
        keyword_code)
    code.putln(
      "%s = %s; if (!%s) %s" % (
        self.result,
        call_code,
        self.result,
        code.error_goto(self.pos)))
    self.function.generate_disposal_code(code)
    self.positional_args.generate_disposal_code(code)
    if self.keyword_args:
      self.keyword_args.generate_disposal_code(code)


class AsTupleNode(ExprNode):
  #  Convert argument to tuple. Used for normalising
  #  the * argument of a function call.
  #
  #  arg    ExprNode
  
  def analyse_expressions(self, env):
    self.arg.analyse_expressions(env)
    self.arg = self.arg.coerce_to_pyobject(env)
    self.type = PyrexTypes.py_object_type
    self.allocate_temp(env)
    self.arg.release_temp(env)
  
  def generate_evaluation_code(self, code):
    self.arg.generate_evaluation_code(code)
    code.putln(
      "%s = PySequence_Tuple(%s); if (!%s) %s" % (
        self.result,
        self.arg.result,
        self.result,
        code.error_goto(self.pos)))
  
  def generate_subexpr_disposal_code(self, code):
    self.arg.generate_disposal_code(code)


class AttributeNode(ExprNode):
  #  obj.attribute
  #
  #  obj          ExprNode
  #  attribute    string
  #
  #  Used internally:
  #
  #  is_py_attr   boolean   Is a Python getattr operation
  #  select_code  string    "<object>.<attribute>"
  
  type = PyrexTypes.error_type
  result = "<error>"
  select_code = "<error>"

  def analyse_target_declaration(self, env):
    pass
  
  def analyse_expressions(self, env, target = 0):
    #print "AttributeNode.analyse_expressions" ###
    self.obj.analyse_expressions(env)
    self.analyse_attribute(env)
    #print "AttributeNode.analyse_expressions: self.type =", self.type ###
    #print "...target =", target ###
    # Reference to C array turns into pointer to first element.
    if self.type.is_array:
      self.type = self.type.element_ptr_type()
    if self.type.is_pyobject:
      if not target:
        #print "AttributeNode.analyse_expressions: allocating temp" ###
        self.allocate_temp(env)
        self.release_subexpr_temps(env)
    else:
      self.result = self.select_code

  def analyse_attribute(self, env):
    # Look up attribute and set self.type and self.select_code.
    #print "AttributeNode.analyse_attribute" ###
    self.is_py_attr = 0
    if self.obj.type.is_string:
      self.obj = self.obj.coerce_to_pyobject(env)
    obj_type = self.obj.type
    #print "...obj_type =", obj_type, "attr =", self.attribute ###
    if obj_type.is_ptr:
      obj_type = obj_type.base_type
      op = "->"
    elif obj_type.is_extension_type:
      op = "->"
    else:
      op = "."
    if obj_type.has_attributes:
      #print "AttributeNode.analyse_attribute: has_attributes" ###
      entry = None
      if obj_type.attributes_known():
        #print "...looking for:", self.attribute ###
        entry = obj_type.scope.lookup_here(self.attribute)
        #print "......found:", entry ###
      else:
        error(self.pos, 
          "Cannot select attribute of incomplete type '%s'" 
          % obj_type)
        obj_type = PyrexTypes.error_type
      if entry:
        if entry.is_variable:
          self.type = entry.type
          obj_code = self.obj.result
          if self.obj.type.is_extension_type:
            obj_code = "((%s)%s)" % (
              obj_type.declaration_code(""), obj_code)
          self.select_code = "%s%s%s" % (
            obj_code, op, self.attribute)
          if self.type.is_extension_type:
            self.select_code = "((PyObject *)%s)" % self.select_code
          return
        else:
          # If it's not a variable, it must be a method of an 
          # extension type, so we treat it like a Python attribute.
          pass
    # If we get here, the base object is not a struct/union/extension 
    # type, or it is an extension type and the attribute is either not
    # declared or is declared as a method. Treat as a Python attribute 
    # reference.
    if obj_type.is_pyobject:
      self.type = PyrexTypes.py_object_type
      self.is_py_attr = 1
    else:
      if not obj_type.is_error:
        error(self.pos, 
          "Object of type '%s' has no attribute '%s'" %
          (obj_type, self.attribute))
    
  def is_lvalue(self):
    return 1
  
  def analyse_target_expression(self, env):
    #print "AttributeNode.analyse_target_expression" ###
    self.analyse_expressions(env, target = 1)
  
  def release_subexpr_temps(self, env):
    self.obj.release_temp(env)
  
  def generate_evaluation_code(self, code):
    self.obj.generate_evaluation_code(code)
    if self.is_py_attr:
      code.putln(
        '%s = PyObject_GetAttrString(%s, "%s"); if (!%s) %s' % (
          self.result,
          self.obj.result,
          self.attribute,
          self.result,
          code.error_goto(self.pos)))
    elif self.type.is_pyobject:
      #print "AttributeNode.generate_evaluation_code: py object ref" ###
      code.put("%s = %s; " % (
        self.result, 
        self.select_code))
      code.put_incref(self.result)
    # else:
    #   Must be a C struct member -- needs no evaluation
    self.obj.generate_disposal_code(code)
  
  def generate_assignment_code(self, rhs, code):
    self.obj.generate_evaluation_code(code)
    if self.is_py_attr:
      code.putln(
        'if (PyObject_SetAttrString(%s, "%s", %s) < 0) %s' % (
          self.obj.result,
          self.attribute,
          rhs.result,
          code.error_goto(self.pos)))
      rhs.generate_disposal_code(code)
    else:
      if self.type.is_pyobject:
        rhs.make_owned_reference(code)
        code.put_decref(self.select_code)
      code.putln(
        "%s = %s;" % (
          self.select_code,
          rhs.result))
      rhs.generate_post_assignment_code(code)
    self.obj.generate_disposal_code(code)
  
  def generate_deletion_code(self, code):
    self.obj.generate_evaluation_code(code)
    if self.is_py_attr:
      code.putln(
        'if (PyObject_DelAttrString(%s, "%s") < 0) %s' % (
          self.obj.result,
          self.attribute,
          code.error_goto(self.pos)))
    else:
      error("Cannot delete C attribute of extension type", self.pos)
    self.obj.generate_disposal_code(code)
  
  def generate_subexpr_disposal_code(self, code):
    self.obj.generate_disposal_code(code)

#-------------------------------------------------------------------
#
#  Constructor nodes
#
#-------------------------------------------------------------------

class SequenceNode(ExprNode):
  #  Base class for list and tuple constructor nodes.
  #  Contains common code for performing sequence unpacking.
  #
  #  args            [ExprNode]
  #  unpacked_item   ExprNode or None
  
  is_sequence_constructor = 1
  unpacked_item = None
  
  def analyse_target_declaration(self, env):
    for arg in self.args:
      arg.analyse_target_declaration(env)

  def analyse_expressions(self, env):
    for i in range(len(self.args)):
      arg = self.args[i]
      arg.analyse_expressions(env)
      self.args[i] = arg.coerce_to_pyobject(env)
    self.type = PyrexTypes.py_object_type
    self.allocate_temp(env)
    self.release_subexpr_temps(env)
  
  def analyse_target_expression(self, env):
    for arg in self.args:
      arg.analyse_target_expression(env)
    #print "UnpackNode.analyse_target_expressions:" ###
    #print "...creating unpacked_item" ###
    self.unpacked_item = PyTempNode(self.pos, env)
    self.type = PyrexTypes.py_object_type
    self.release_subexpr_temps(env)
    #env.global_scope().uses_unpacking = 1
    env.use_utility_code(unpacking_utility_code)
  
  def release_subexpr_temps(self, env):
    for arg in self.args:
      arg.release_temp(env)
  
  def release_temp(self, env):
    if self.unpacked_item:
      self.unpacked_item.release_temp(env)
    ExprNode.release_temp(self, env)
  
  def generate_evaluation_code(self, code):
    for arg in self.args:
      arg.generate_evaluation_code(code)
    self.generate_operation_code(code)
    if debug_disposal_code:
      print "UnpackNode.generate_evaluation_code:"
      print "...generating subexpr disposal code for", self
    self.generate_subexpr_disposal_code(code)
  
  def generate_assignment_code(self, rhs, code):
    for i in range(len(self.args)):
      code.putln(
        "%s = __Pyx_UnpackItem(%s, %s); if (!%s) %s" % (
          self.unpacked_item.result,
          rhs.result,
          i,
          self.unpacked_item.result,
          code.error_goto(self.pos)))
      self.args[i].generate_assignment_code(
        self.unpacked_item, code)
    code.putln(
      "if (__Pyx_EndUnpack(%s, %s) < 0) %s" % (
        rhs.result,
        len(self.args),
        code.error_goto(self.pos)))
    if debug_disposal_code:
      print "UnpackNode.generate_assignment_code:"
      print "...generating disposal code for", rhs
    rhs.generate_disposal_code(code)

  def generate_subexpr_disposal_code(self, code):
    for arg in self.args:
      if debug_disposal_code:
        print "UnpackNode.generate_subexpr_disposal_code"
        print "...generating disposal code for", arg
      arg.generate_disposal_code(code)


class TupleNode(SequenceNode):
  #  Tuple constructor.

  def generate_operation_code(self, code):
    code.putln(
      "%s = PyTuple_New(%s); if (!%s) %s" % (
        self.result,
        len(self.args),
        self.result,
        code.error_goto(self.pos)))
    for i in range(len(self.args)):
      arg = self.args[i]
      if not arg.is_temp:
        code.put_incref(arg.result)
      code.putln(
        "PyTuple_SET_ITEM(%s, %s, %s);" % (
          self.result,
          i,
          arg.result))
  
  def generate_subexpr_disposal_code(self, code):
    # We call generate_post_assignment_code here instead
    # of generate_disposal_code, because values were stored
    # in the tuple using a reference-stealing operation.
    for arg in self.args:
      if debug_disposal_code:
        print "TupleNode.generate_subexpr_disposal_code"
        print "...generating post-assignment code for", arg
      arg.generate_post_assignment_code(code)		


class ListNode(SequenceNode):
  #  List constructor.
  
  def generate_operation_code(self, code):
    code.putln("%s = PyList_New(%s); if (!%s) %s" %
      (self.result,
      len(self.args),
      self.result,
      code.error_goto(self.pos)))
    for i in range(len(self.args)):
      arg = self.args[i]
      if not arg.is_temp:
        code.put_incref(arg.result)
      code.putln("PyList_SET_ITEM(%s, %s, %s);" %
        (self.result,
        i,
        arg.result))
        
  def generate_subexpr_disposal_code(self, code):
    # We call generate_post_assignment_code here instead
    # of generate_disposal_code, because values were stored
    # in the list using a reference-stealing operation.
    for arg in self.args:
      if debug_disposal_code:
        print "ListNode.generate_subexpr_disposal_code"
        print "...generating post-assignment code for", arg
      arg.generate_post_assignment_code(code)		


class DictNode(ExprNode):
  #  Dictionary constructor.
  #
  #  key_value_pairs  [(ExprNode, ExprNode)]
  
  def analyse_expressions(self, env):
    self.type = PyrexTypes.py_object_type
    self.allocate_temp(env)
    new_pairs = []
    for key, value in self.key_value_pairs:
      key.analyse_expressions(env)
      value.analyse_expressions(env)
      key = key.coerce_to_pyobject(env)
      value = value.coerce_to_pyobject(env)
      new_pairs.append((key, value))
      key.release_temp(env)
      value.release_temp(env)
    self.key_value_pairs = new_pairs
  
  def generate_evaluation_code(self, code):
    code.putln(
      "%s = PyDict_New(); if (!%s) %s" % (
        self.result,
        self.result,
        code.error_goto(self.pos)))
    for key, value in self.key_value_pairs:
      key.generate_evaluation_code(code)
      value.generate_evaluation_code(code)
      code.putln(
        "if (PyDict_SetItem(%s, %s, %s) < 0) %s" % (
          self.result,
          key.result,
          value.result,
          code.error_goto(self.pos)))
      key.generate_disposal_code(code)
      value.generate_disposal_code(code)
  
  def generate_subexpr_disposal_code(self, code):
    # Should never be called, because result is always temp
    raise InternalError(
      "DictNode.generate_subexpr_disposal_code called")


class ClassNode(ExprNode):
  #  Helper class used in the implementation of Python
  #  class definitions. Constructs a class object given
  #  a name, tuple of bases and class dictionary.
  #
  #  name       ExprNode              Name of the class
  #  bases      ExprNode              Base class tuple
  #  dict       ExprNode              Class dict (not owned by this node)
  #  doc        ExprNode or None
  
  def analyse_expressions(self, env):
    self.name.analyse_expressions(env)
    self.name = self.name.coerce_to_pyobject(env)
    self.bases.analyse_expressions(env)
    if self.doc:
      self.doc.analyse_expressions(env)
      self.doc = self.doc.coerce_to_pyobject(env)
    self.type = PyrexTypes.py_object_type
    self.allocate_temp(env)
    self.name.release_temp(env)
    self.bases.release_temp(env)
    if self.doc:
      self.doc.release_temp(env)
  
  def generate_evaluation_code(self, code):
    self.name.generate_evaluation_code(code)
    self.bases.generate_evaluation_code(code)
    if self.doc:
      self.doc.generate_evaluation_code(code)
    if self.doc:
      code.putln(
        'if (PyDict_SetItemString(%s, "__doc__", %s) < 0) %s' % (
          self.dict.result,
          self.doc.result,
          code.error_goto(self.pos)))
    code.putln(
      '%s = PyClass_New(%s, %s, %s); if (!%s) %s' % (
        self.result,
        self.bases.result,
        self.dict.result,
        self.name.result,
        self.result,
        code.error_goto(self.pos)))
#		if self.doc:
#			code.putln(
#				'if (PyObject_SetAttrString(%s, "__doc__", %s) < 0) %s' % (
#					self.result,
#					self.doc.result,
#					code.error_goto(self.pos)))
    self.name.generate_disposal_code(code)
    self.bases.generate_disposal_code(code)
    if self.doc:
      self.doc.generate_disposal_code(code)


class UnboundMethodNode(ExprNode):
  #  Helper class used in the implementation of Python
  #  class definitions. Constructs an unbound method
  #  object from a class and a function.
  #
  #  class_cname   string     C var holding the class object
  #  function      ExprNode   Function object
  
  def analyse_expressions(self, env):
    self.function.analyse_expressions(env)
    self.type = PyrexTypes.py_object_type
    self.allocate_temp(env)
    self.function.release_temp(env)
  
  def generate_evaluation_code(self, code):
    self.function.generate_evaluation_code(code)
    code.putln(
      "%s = PyMethod_New(%s, 0, %s); if (!%s) %s" % (
        self.result,
        self.function.result,
        self.class_cname,
        self.result,
        code.error_goto(self.pos)))
    self.function.generate_disposal_code(code)


class PyCFunctionNode(ExprNode):
  #  Helper class used in the implementation of Python
  #  class definitions. Constructs a PyCFunction object
  #  from a PyMethodDef struct.
  #
  #  pymethdef_cname   string   PyMethodDef structure
  
  def analyse_expressions(self, env):
    self.type = PyrexTypes.py_object_type
    self.allocate_temp(env)
  
  def generate_evaluation_code(self, code):
    code.putln(
      "%s = PyCFunction_New(&%s, 0); if (!%s) %s" % (
        self.result,
        self.pymethdef_cname,
        self.result,
        code.error_goto(self.pos)))

#-------------------------------------------------------------------
#
#  Unary operator nodes
#
#-------------------------------------------------------------------

class UnopNode(ExprNode):
  #  operator     string
  #  operand      ExprNode
  #
  #  Processing during analyse_expressions phase:
  #
  #    analyse_c_operation
  #      Called when the operand is not a pyobject.
  #      - Check operand type and coerce if needed.
  #      - Determine result type and result code fragment.
  #      - Allocate temporary for result if needed.
  #
  
  def analyse_expressions(self, env):
    self.operand.analyse_expressions(env)
    if self.is_py_operation():
      self.coerce_operand_to_pyobject(env)
      self.type = PyrexTypes.py_object_type
      self.allocate_temp(env)
    else:
      self.analyse_c_operation(env)
    if self.is_temp:
      self.release_subexpr_temps(env)
  
  def check_const(self):
    self.operand.check_const()
  
  def is_py_operation(self):
    return self.operand.type.is_pyobject
  
  def coerce_operand_to_pyobject(self, env):
    self.operand = self.operand.coerce_to_pyobject(env)
  
  def release_subexpr_temps(self, env):
    self.operand.release_temp(env)
  
  def generate_evaluation_code(self, code):
    self.operand.generate_evaluation_code(code)
    if self.operand.type.is_pyobject:
      self.generate_py_operation_code(code)
    else:
      if self.is_temp:
        self.generate_c_operation_code(code)
    if self.is_temp:
      if debug_disposal_code:
        print "UnopNode.generate_evaluation_code:"
        print "...generating subexpr disposal code for", self
      self.generate_subexpr_disposal_code(code)
  
  def generate_py_operation_code(self, code):
    function = self.py_operation_function()
    code.putln(
      "%s = %s(%s); if (!%s) %s" % (
        self.result, 
        function, 
        self.operand.result,
        self.result,
        code.error_goto(self.pos)))
    
  def generate_subexpr_disposal_code(self, code):
    if debug_disposal_code:
      print "UnopNode.generate_subexpr_disposal_code:"
      print "...generating disposal code for", self.operand
    self.operand.generate_disposal_code(code)
  
  def type_error(self):
    if not self.operand.type.is_error:
      error(self.pos, "Invalid operand type for '%s' (%s)" %
        (self.operator, self.operand.type))
    self.type = PyrexTypes.error_type


class NotNode(ExprNode):
  #  'not' operator
  
  def analyse_expressions(self, env):
    self.operand.analyse_expressions(env)
    self.operand = self.operand.coerce_to_boolean(env)
    self.type = PyrexTypes.c_int_type
    self.result = "(!%s)" % self.operand.result
  
  def release_subexpr_temps(self, env):
    self.operand.release_temp(env)
  
  def generate_evaluation_code(self, code):
    self.operand.generate_evaluation_code(code)
  
  def generate_subexpr_disposal_code(self, code):
    self.operand.generate_disposal_code(self)


class UnaryPlusNode(UnopNode):
  #  unary '+' operator
  
  operator = '+'
  
  def analyse_c_operation(self, env):
    self.type = self.operand.type
    self.result = self.operand.result
  
  def py_operation_function(self):
    return "PyNumber_Positive"


class UnaryMinusNode(UnopNode):
  #  unary '-' operator
  
  operator = '-'
  
  def analyse_c_operation(self, env):
    if self.operand.type.is_numeric:
      self.type = self.operand.type
    else:
      self.type_error()
    self.result = "(-%s)" % self.operand.result
  
  def py_operation_function(self):
    return "PyNumber_Negative"


class TildeNode(UnopNode):

  def analyse_c_operation(self, env):
    if self.operand.type.is_int:
      self.type = self.operand.type
    else:
      self.type_error()
    self.result = "(~%s)" % self.operand.result

  def py_operation_function(self):
    return "PyNumber_Invert"


class AmpersandNode(ExprNode):
  #  The C address-of operator.
  #
  #  operand  ExprNode

  def analyse_expressions(self, env):
    self.operand.analyse_expressions(env)
    argtype = self.operand.type
    if not (argtype.is_cfunction or self.operand.is_lvalue()):
      self.error("Taking address of non-lvalue")
      return
    if argtype.is_pyobject:
      self.error("Cannot take address of Python variable")
      return
    self.type = PyrexTypes.c_ptr_type(argtype)
    self.result = "(&%s)" % self.operand.result
  
  def check_const(self):
    #print "AmpersandNode.check_const" ###
    self.operand.check_addr_const()
  
  def release_subexpr_temps(self, env):
    self.operand.release_temp(env)
  
  def error(self, mess):
    error(self.pos, mess)
    self.type = PyrexTypes.error_type
    self.result = "<error>"
  
  def generate_evaluation_code(self, code):
    self.operand.generate_evaluation_code(code)
  
  def generate_subexpr_disposal_code(self, code):
    self.operand.generate_disposal_code(code)


unop_node_classes = {
  "+":  UnaryPlusNode,
  "-":  UnaryMinusNode,
  "~":  TildeNode,
}

def unop_node(pos, operator, operand):
  # Construct unnop node of appropriate class for 
  # given operator.
  return unop_node_classes[operator](pos, 
    operator = operator, 
    operand = operand)


class TypecastNode(ExprNode):
  #  C type cast
  #
  #  base_type    CBaseTypeNode
  #  declarator   CDeclaratorNode
  #  operand      ExprNode
  
  def analyse_expressions(self, env):
    base_type = self.base_type.analyse(env)
    name, self.type = self.declarator.analyse(base_type, env)
    self.operand.analyse_expressions(env)
    to_py = self.type.is_pyobject
    from_py = self.operand.type.is_pyobject
    if from_py and not to_py and self.operand.is_ephemeral():
      error(self.pos, "Casting temporary Python object to non-Python type")
    if to_py:
      self.allocate_temp(env)
    else:
      self.result = "((%s)%s)" % (
        self.type.declaration_code(""),
        self.operand.result)
  
  def check_const(self):
    self.operand.check_const()
  
  def release_subexpr_temps(self, env):
    self.operand.release_temp(env)
  
  def generate_evaluation_code(self, code):
    self.operand.generate_evaluation_code(code)
    if self.type.is_pyobject:
      code.putln(
        "%s = (%s)%s;" % (
          self.result,
          self.type.declaration_code(""),
          self.operand.result))
      code.put_incref(self.result)

  def generate_subexpr_disposal_code(self, code):
    self.operand.generate_disposal_code(code)


class SizeofNode(ExprNode):
  #  Base class for sizeof(x) expression nodes.

  def check_const(self):
    pass
  
  def generate_evaluation_code(self, code):
    pass
  
  def generate_subexpr_disposal_code(self, code):
    pass


class SizeofTypeNode(SizeofNode):
  #  C sizeof function applied to a type
  #
  #  base_type   CBaseTypeNode
  #  declarator  CDeclaratorNode
  
  def analyse_expressions(self, env):
    base_type = self.base_type.analyse(env)
    _, arg_type = self.declarator.analyse(base_type, env)
    arg_code = arg_type.declaration_code("")
    if arg_type.is_pyobject:
      error(self.pos, "Cannot take sizeof Python object")
    elif arg_type.is_void:
      error(self.pos, "Cannot take sizeof void")
    elif not arg_type.is_complete():
      error(self.pos, "Cannot take sizeof incomplete type '%s'" % arg_code)
    self.type = PyrexTypes.c_int_type
    self.result = "(sizeof(%s))" % arg_code

  def release_subexpr_temps(self, env):
    pass


class SizeofVarNode(SizeofNode):
  #  C sizeof function applied to a variable
  #
  #  operand   ExprNode
  
  def analyse_expressions(self, env):
    self.operand.analyse_expressions(env)
    if self.operand.type.is_pyobject:
      error(self.pos, "Cannot take sizeof Python object")
    self.type = PyrexTypes.c_int_type
    self.result = "(sizeof %s)" % self.operand.result
  
  def release_subexpr_temps(self, env):
    self.operand.release_temp(env)
  

#-------------------------------------------------------------------
#
#  Binary operator nodes
#
#-------------------------------------------------------------------

class BinopNode(ExprNode):
  #  operator     string
  #  operand1     ExprNode
  #  operand2     ExprNode
  #
  #  Processing during analyse_expressions phase:
  #
  #    analyse_c_operation
  #      Called when neither operand is a pyobject.
  #      - Check operand types and coerce if needed.
  #      - Determine result type and result code fragment.
  #      - Allocate temporary for result if needed.
  #
  
  def analyse_expressions(self, env):
    #print "BinopNode.analyse_expressions: analysing operand1" ###
    self.operand1.analyse_expressions(env)
    #print "BinopNode.analyse_expressions: analysing operand2" ###
    self.operand2.analyse_expressions(env)
    if self.is_py_operation():
      self.coerce_operands_to_pyobjects(env)
      self.type = PyrexTypes.py_object_type
      self.allocate_temp(env)
    else:
      self.analyse_c_operation(env)
    if self.is_temp:
      #print "BinopNode.analyse_expressions: releasing subexpr temps" ###
      self.release_subexpr_temps(env)
  
  def check_const(self):
    self.operand1.check_const()
    self.operand2.check_const()
  
  def is_py_operation(self):
    return (self.operand1.type.is_pyobject 
      or self.operand2.type.is_pyobject)
  
  def coerce_operands_to_pyobjects(self, env):
    #print "BinopNode.coerce_operands_to_pyobjects: coercing operand1" ###
    self.operand1 = self.operand1.coerce_to_pyobject(env)
    #print "BinopNode.coerce_operands_to_pyobjects: coercing operand2" ###
    self.operand2 = self.operand2.coerce_to_pyobject(env)
  
  def release_subexpr_temps(self, env):
    self.operand1.release_temp(env)
    self.operand2.release_temp(env)
  
  def generate_evaluation_code(self, code):
    self.operand1.generate_evaluation_code(code)
    self.operand2.generate_evaluation_code(code)
    if self.operand1.type.is_pyobject:
      function = self.py_operation_function()
      if function == "PyNumber_Power":
        extra_args = ", Py_None"
      else:
        extra_args = ""
      code.putln(
        "%s = %s(%s, %s%s); if (!%s) %s" % (
          self.result, 
          function, 
          self.operand1.result,
          self.operand2.result,
          extra_args,
          self.result,
          code.error_goto(self.pos)))
    else:
      if self.is_temp:
        self.generate_c_operation_code(code)
    if self.is_temp:
      if debug_disposal_code:
        print "BinopNode.generate_evaluation_code:"
        print "...generating subexpr disposal code for", self
      self.generate_subexpr_disposal_code(code)
    
  def generate_subexpr_disposal_code(self, code):
    if debug_disposal_code:
      print "BinopNode.generate_subexpr_disposal_code:"
      print "...generating disposal code for", self.operand1
      print "...and", self.operand2
    self.operand1.generate_disposal_code(code)
    self.operand2.generate_disposal_code(code)
  
  def type_error(self):
    if not (self.operand1.type.is_error
        or self.operand2.type.is_error):
      error(self.pos, "Invalid operand types for '%s' (%s; %s)" %
        (self.operator, self.operand1.type, 
          self.operand2.type))
    self.type = PyrexTypes.error_type


class NumBinopNode(BinopNode):
  #  Binary operation taking numeric arguments.
  
  def analyse_c_operation(self, env):
    type1 = self.operand1.type
    type2 = self.operand2.type
    self.type = self.compute_c_result_type(type1, type2)
    if not self.type:
      self.type_error()
    self.result = self.compute_c_result()
  
  def compute_c_result_type(self, type1, type2):
    if self.c_types_okay(type1, type2):
      return PyrexTypes.widest_numeric_type(type1, type2)
    else:
      return None
  
  def compute_c_result(self):
    return "(%s %s %s)" % (
      self.operand1.result, 
      self.operator, 
      self.operand2.result)
  
  def c_types_okay(self, type1, type2):
    return type1.is_numeric and type2.is_numeric

  def py_operation_function(self):
    return self.py_functions[self.operator]

  py_functions = {
    "|":		"PyNumber_Or",
    "^":		"PyNumber_Xor",
    "&":		"PyNumber_And",
    "<<":		"PyNumber_Lshift",
    ">>":		"PyNumber_Rshift",
    "+":		"PyNumber_Add",
    "-":		"PyNumber_Subtract",
    "*":		"PyNumber_Multiply",
    "/":		"PyNumber_Divide",
    "%":		"PyNumber_Remainder",
    "**":   "PyNumber_Power"
  }


class IntBinopNode(NumBinopNode):
  #  Binary operation taking integer arguments.
  
  def c_types_okay(self, type1, type2):
    return type1.is_int and type2.is_int

  
class AddNode(NumBinopNode):
  #  '+' operator.
  
  def is_py_operation(self):
    if self.operand1.type.is_string \
      and self.operand2.type.is_string:
        return 1
    else:
      return NumBinopNode.is_py_operation(self)

  def compute_c_result_type(self, type1, type2):
    if type1.is_ptr and type2.is_int:
      return type1
    elif type1.is_int and type2.is_ptr:
      return type2
    else:
      return NumBinopNode.compute_c_result_type(
        self, type1, type2)


class SubNode(NumBinopNode):
  #  '-' operator.
  
  def compute_c_result_type(self, type1, type2):
    if type1.is_ptr and type2.is_int:
      return type1
    elif type1.is_ptr and type2.is_ptr:
      return PyrexTypes.c_int_type
    else:
      return NumBinopNode.compute_c_result_type(
        self, type1, type2)


class MulNode(NumBinopNode):
  #  '*' operator.
  
  def is_py_operation(self):
    type1 = self.operand1.type
    type2 = self.operand2.type
    if (type1.is_string and type2.is_int) \
      or (type2.is_string and type1.is_int):
        return 1
    else:
      return NumBinopNode.is_py_operation(self)


class ModNode(IntBinopNode):
  #  '%' operator.
  
  def is_py_operation(self):
    #print "ModNode.is_py_operation:", \
    #	self.operand1.type, self.operand2.type ###
    return (self.operand1.type.is_string
      or self.operand2.type.is_string
      or IntBinopNode.is_py_operation(self))


class PowNode(NumBinopNode):
  #  '**' operator.
  
  def analyse_expressions(self, env):
    #env.global_env().math_library_used = 1
    env.pow_function_used = 1
    NumBinopNode.analyse_expressions(self, env)
  
  def compute_c_result_type(self, type1, type2):
    if self.c_types_okay(type1, type2):
      return PyrexTypes.c_double_type
    else:
      return None
  
  def compute_c_result(self):
    return "pow(%s, %s)" % (
      self.operand1.result, self.operand2.result)
      

class BoolBinopNode(ExprNode):
  #  Short-circuiting boolean operation.
  #
  #  operator     string
  #  operand1     ExprNode
  #  operand2     ExprNode
  #  temp_bool    string       used internally
  
  def analyse_expressions(self, env):
    self.operand1.analyse_expressions(env)
    self.operand2.analyse_expressions(env)
    if (self.operand1.type.is_pyobject or 
      self.operand2.type.is_pyobject):
        self.operand1 = self.operand1.coerce_to_pyobject(env)
        self.operand2 = self.operand2.coerce_to_pyobject(env)
        self.operand1 = self.operand1.coerce_to_temp(env)
        self.operand2 = self.operand2.coerce_to_temp(env)
        self.temp_bool = env.allocate_temp(PyrexTypes.c_int_type)
        # Force both operands to share the same temp result var,
        # which also becomes the result of this node.
        self.type = PyrexTypes.py_object_type
        self.result = self.operand2.result
        self.is_temp = 1
        env.release_temp(self.temp_bool)
        self.operand1.release_temp(env)
        self.operand1.result = self.result
    else:
      self.operand1 = self.operand1.coerce_to_boolean(env)
      self.operand2 = self.operand2.coerce_to_boolean(env)
      self.type = PyrexTypes.c_int_type
      self.allocate_temp(env)
      self.release_subexpr_temps(env)
  
  def check_const(self):
    self.operand1.check_const()
    self.operand2.check_const()

  def release_subexpr_temps(self, env):
    if self.operand1.result <> self.result:
      self.operand1.release_temp(env)
    if self.operand2.result <> self.result:
      self.operand2.release_temp(env)
  
  def generate_evaluation_code(self, code):
    if self.type.is_pyobject:
      self.generate_py_evaluation_code(code)
    else:
      self.generate_c_evaluation_code(code)
  
  def generate_py_evaluation_code(self, code):
      self.operand1.generate_evaluation_code(code)
      code.putln(
        "%s = PyObject_IsTrue(%s); if (%s < 0) %s" % (
          self.temp_bool,
          self.result,
          self.temp_bool,
          code.error_goto(self.pos)))
      if self.operator == 'and':
        test = "if (%s) {"
      else:
        test = "if (!%s) {"
      code.putln(
        test % self.temp_bool)
      self.operand1.generate_disposal_code(code)
      self.operand2.generate_evaluation_code(code)
      code.putln(
        "}")
  
  def generate_c_evaluation_code(self, code):
    #print "BoolBinopNode.generate_evaluation_code:" ###
    #print "...generating evaluation code for operand1 (%s)" % self.operand1 ###
    self.operand1.generate_evaluation_code(code)
    if self.operator == 'and':
      test = "if (%s = %s) {"
    else: # self.operator == 'or'
      test = "if (!(%s = %s)) {"
    code.putln(test % (self.result, self.operand1.result))
    #print "BoolBinopNode.generate_evaluation_code:" ###
    #print "...generating evaluation code for operand2 (%s)" % self.operand2 ###
    self.operand2.generate_evaluation_code(code)
    code.putln("%s = %s;" % (
      self.result, self.operand2.result))
    self.operand2.generate_disposal_code(code)
    code.putln("}")
    self.operand1.generate_disposal_code(code)
    

class CmpNode:
  #  Mixin class containing code common to PrimaryCmpNodes
  #  and CascadedCmpNodes.
  
  def is_python_comparison(self):
    return (self.has_python_operands()
      or (self.cascade and self.cascade.is_python_comparison())
      or self.operator in ('in', 'not_in'))

  def check_types(self, env, operand1, op, operand2):
    if not self.types_okay(operand1, op, operand2):
      error(self.pos, "Invalid types for '%s' (%s, %s)" %
        (self.operator, operand1.type, operand2.type))
  
  def types_okay(self, operand1, op, operand2):
    type1 = operand1.type
    type2 = operand2.type
    if type1.is_error or type2.is_error:
      return 1
    if type1.is_pyobject: # type2 will be, too
      return 1
    elif type1.is_ptr:
      return type1.is_null_ptr or type2.is_null_ptr \
        or type1.same_as(type2)
    elif (type1.is_numeric and type2.is_numeric
        and op not in ('is', 'is_not')):
      return 1
    else:
      return 0

  def generate_operation_code(self, code, result, 
      operand1, op , operand2):
    #print "CmpNode.generate_operation_code: op =", repr(op) ###
    if op == 'in' or op == 'not_in':
      code.putln(
        "%s = PySequence_Contains(%s, %s); if (%s < 0) %s" % (
          result, 
          operand2.result, 
          operand1.result, 
          result,
          code.error_goto(self.pos)))
      if op == 'not_in':
        code.putln(
          "%s = !%s;" % (
            result, result))
    elif (operand1.type.is_pyobject
      and op not in ('is', 'is_not')):
        code.putln(
          "if (PyObject_Cmp(%s, %s, &%s) < 0) %s" % (
            operand1.result, 
            operand2.result, 
            result,
            code.error_goto(self.pos)))
        code.putln(
          "%s = %s %s 0;" % (
            result, result, op))
    else:
      code.putln("%s = %s %s %s;" % (
        result, 
        operand1.result, 
        self.c_operator(op), 
        operand2.result))
  
  def c_operator(self, op):
    if op == 'is':
      return "=="
    elif op == 'is_not':
      return "!="
    else:
      return op
  

class PrimaryCmpNode(ExprNode, CmpNode):
  #  Non-cascaded comparison or first comparison of
  #  a cascaded sequence.
  #
  #  operator      string
  #  operand1      ExprNode
  #  operand2      ExprNode
  #  cascade       CascadedCmpNode
  
  cascade = None
  
  def analyse_expressions(self, env):
    self.operand1.analyse_expressions(env)
    self.operand2.analyse_expressions(env)
    if self.cascade:
      self.operand2 = self.operand2.coerce_to_temp(env)
      self.cascade.analyse_expressions(env, self.operand2)
    is_pycmp = self.is_python_comparison()
    if is_pycmp:
      #print "PrimaryCmpNode.analyse_expressions: coercing operands to pyobjects" ###
      self.coerce_operands_to_pyobjects(env)
    self.check_operand_types(env)
    self.type = PyrexTypes.c_int_type
    if is_pycmp or self.cascade:
      self.allocate_temp(env)
      self.operand1.release_temp(env)
      self.operand2.release_temp(env)
    else:
      self.result = "(%s %s %s)" % (
        self.operand1.result,
        self.c_operator(self.operator),
        self.operand2.result)
  
  def check_const(self):
    self.operand1.check_const()
    self.operand2.check_const()

  def check_operand_types(self, env):
    self.check_types(env, 
      self.operand1, self.operator, self.operand2)
    if self.cascade:
      self.cascade.check_operand_types(env, self.operand2)
  
  def has_python_operands(self):
    return (self.operand1.type.is_pyobject
      or self.operand2.type.is_pyobject)
  
  def coerce_operands_to_pyobjects(self, env):
    self.operand1 = self.operand1.coerce_to_pyobject(env)
    self.operand2 = self.operand2.coerce_to_pyobject(env)
    if self.cascade:
      self.cascade.coerce_operands_to_pyobjects(env)
    
  def release_subexpr_temps(self, env):
    self.operand1.release_temp(env)
    self.operand2.release_temp(env)
    # If this method is called, it means that the result of
    # this node is not temporary. Therefore, there cannot be
    # any cascaded cmps attached.

  def generate_evaluation_code(self, code):
    #print "PrimaryCmpNode.generate_evaluation_code" ###
    self.operand1.generate_evaluation_code(code)
    self.operand2.generate_evaluation_code(code)
    if self.is_temp:
      self.generate_operation_code(code, self.result, 
        self.operand1, self.operator, self.operand2)
      if self.cascade:
        self.cascade.generate_evaluation_code(code,
          self.result, self.operand2)
      if debug_disposal_code:
        print "PrimaryCmpNode.generate_evaluation_code:"
        print "...generating subexpr disposal code for", self
      self.generate_subexpr_disposal_code(code)
  
  def generate_subexpr_disposal_code(self, code):
    if debug_disposal_code:
      print "PrimaryCmpNode.generate_subexpr_disposal_code:"
      print "...generating disposal code for", self.operand1
      print "...and", self.operand2
    self.operand1.generate_disposal_code(code)
    self.operand2.generate_disposal_code(code)
      

class CascadedCmpNode(Node, CmpNode):
  #  A CascadedCmpNode is not a complete expression node. It 
  #  hangs off the side of another comparison node, shares 
  #  its left operand with that node, and shares its result 
  #  with the PrimaryCmpNode at the head of the chain.
  #
  #  operator      string
  #  operand2      ExprNode
  #  cascade       CascadedCmpNode

  cascade = None
  
  def analyse_expressions(self, env, operand1):
    self.operand2.analyse_expressions(env)
    if self.cascade:
      self.operand2 = self.operand2.coerce_to_temp(env)
      self.cascade.analyse_expressions(env, self.operand2)
    # Cascaded cmp result is always temp
    self.operand2.release_temp(env)

  def check_const(self):
    self.operand2.check_const()

  def check_operand_types(self, env, operand1):
    self.check_types(env, 
      operand1, self.operator, self.operand2)
    if self.cascade:
      self.cascade.check_operand_types(env, self.operand2)
  
  def has_python_operands(self):
    return self.operand2.type.is_pyobject

  def coerce_operands_to_pyobjects(self, env):
    self.operand2 = self.operand2.coerce_to_pyobject(env)
    if self.cascade:
      self.cascade.coerce_operands_to_pyobjects(env)

  def generate_evaluation_code(self, code, result, operand1):
    #print "CascadedCmpNode.generate_evaluation_code" ###
    code.putln("if (%s) {" % result)
    self.operand2.generate_evaluation_code(code)
    self.generate_operation_code(code, result, 
      operand1, self.operator, self.operand2)
    if self.cascade:
      self.cascade.generate_evaluation_code(
        code, result, self.operand2)
    # Cascaded cmp result is always temp
    if debug_disposal_code:
      print "CascadedCmpNode.generate_evaluation_code:"
      print "...generating subexpr disposal code for", self
    #self.operand2.generate_disposal_code(code)
    self.generate_subexpr_disposal_code(code)
    code.putln("}")

  def generate_subexpr_disposal_code(self, code):
    if debug_disposal_code:
      print "CascadedCmpNode.generate_subexpr_disposal_code:"
      print "...generating disposal code for", self.operand2
    #self.operand2.generate_subexpr_disposal_code(code)
    self.operand2.generate_disposal_code(code)


binop_node_classes = {
  "or":		BoolBinopNode,
  "and":	BoolBinopNode,
  "|":		IntBinopNode,
  "^":		IntBinopNode,
  "&":		IntBinopNode,
  "<<":		IntBinopNode,
  ">>":		IntBinopNode,
  "+":		AddNode,
  "-":		SubNode,
  "*":		MulNode,
  "/":		NumBinopNode,
  "%":		ModNode,
  "**":		PowNode
}

def binop_node(pos, operator, operand1, operand2):
  # Construct binop node of appropriate class for 
  # given operator.
  return binop_node_classes[operator](pos, 
    operator = operator, 
    operand1 = operand1, 
    operand2 = operand2)

#-------------------------------------------------------------------
#
#  Coercion nodes
#
#  Coercion nodes are special in that they are created during
#  the analyse_expressions phase of parse tree processing.
#  Their __init__ methods consequently incorporate some aspects
#  of that phase.
#
#-------------------------------------------------------------------

class CoercionNode(ExprNode):
  #  Base class for coercion nodes.
  #
  #  arg       ExprNode       node being coerced
  
  def __init__(self, arg):
    self.pos = arg.pos
    self.arg = arg
  
  def release_subexpr_temps(self, env):
    self.arg.release_subexpr_temps(env)
  
  def generate_subexpr_disposal_code(self, code):
    if debug_disposal_code:
      print "CoercionNode.generate_subexpr_disposal_code:"
      print "...generating disposal code for", self.arg
    #self.arg.generate_subexpr_disposal_code(code)
    self.arg.generate_disposal_code(code)


#class CastToPyObjectNode(CoercionNode):
#
#	def __init__(self, arg, env):
#		#  This node is used to cast an extension type
#		#  to a generic Python object. 
#		#  Note: This node "steals" the result from its
#		#  argument node.
#		self.arg = arg
#		self.type = PyrexTypes.py_object_type
#		self.result = "((PyObject *)%s)" % arg.result
#		self.is_temp = arg.is_temp
#		arg.is_temp = 0
#		
#		def generate_evaluation_code(self, code):
#			pass


class PyTypeTestNode(CoercionNode):

  def __init__(self, arg, dst_type, env):
    #  The arg is know to be a Python object, and
    #  the dst_type is known to be an extension type.
    #  Note: This node "steals" the result from its
    #  argument node.
    CoercionNode.__init__(self, arg)
    self.type = dst_type
    #self.result = "((%s)%s)" % (dst_type.declaration_code(""), arg.result)
    self.result = arg.result
    self.is_temp = arg.is_temp
    arg.is_temp = 0
    env.use_utility_code(type_test_utility_code)
  
  def generate_evaluation_code(self, code):
    self.arg.generate_evaluation_code(code)
    typeobj_cname = self.type.typeobj_cname
    if typeobj_cname:
      code.putln(
        "if (!__Pyx_TypeTest(%s, &%s)) %s" % (
          self.result,
          typeobj_cname,
          code.error_goto(self.pos)))
    else:
      error(self.pos, "Cannot test type of extern C class "
        "without type object name specification")
        
        
class CoerceToPyTypeNode(CoercionNode):

  def __init__(self, arg, env):
    CoercionNode.__init__(self, arg)
    self.type = PyrexTypes.py_object_type
    if not arg.type.to_py_function:
      error(arg.pos,
        "Cannot convert '%s' to Python object" % arg.type)
    self.allocate_temp(env)
    arg.release_temp(env)
  
  def generate_evaluation_code(self, code):
    self.arg.generate_evaluation_code(code)
    #print "CoerceToPyTypeNode.generate_evaluation_code:" ###
    #print "...generating evaluation code for arg (%s)" % self.arg ###
    function = self.arg.type.to_py_function
    code.putln('%s = %s(%s); if (!%s) %s' % (
      self.result, 
      function, 
      self.arg.result, 
      self.result, 
      code.error_goto(self.pos)))
    if debug_disposal_code:
      print "CoerceToPyTypeNode.generate_evaluation_code:"
      print "...generating subexpr disposal code for", self
    #self.arg.generate_disposal_code(code)
    self.generate_subexpr_disposal_code(code)


class CoerceFromPyTypeNode(CoercionNode):

  def __init__(self, result_type, arg, env):
    CoercionNode.__init__(self, arg)
    self.type = result_type
    if not result_type.from_py_function:
      error(arg.pos,
        "Cannot convert Python object to '%s'" % result_type)
    if self.type.is_string and self.arg.is_temp:
      error(arg.pos,
        "Obtaining char * from temporary Python value")
    self.allocate_temp(env)
    arg.release_temp(env)
  
  def generate_evaluation_code(self, code):
    #print "CoerceFromPyTypeNode.generate_evaluation_code:" ###
    #print "...generating evaluation code for arg (%s)" % self.arg ###
    self.arg.generate_evaluation_code(code)
    opnd = self.arg.result
    function = self.type.from_py_function
    code.putln('%s = %s(%s); if (PyErr_Occurred()) %s' % (
      self.result, 
      function, 
      self.arg.result, 
      code.error_goto(self.pos)))
    if debug_disposal_code:
      print "CoerceFromPyTypeNode.generate_evaluation_code:"
      print "...generating subexpr disposal code for", self
    self.generate_subexpr_disposal_code(code)
    
#		# If we have just converted a Python string to a char *,
#		# defer deallocating the Python string until the result
#		# has been used.
#		if not self.type.is_string:
#			if debug_disposal_code:
#				print "CoerceFromPyTypeNode.generate_evaluation_code:"
#				print "...generating subexpr disposal code for", self
#			#self.arg.generate_disposal_code(code)
#			self.generate_subexpr_disposal_code(code)
  
  def generate_disposal_code(self, code):
    #print "CoerceFromPyTypeNode.generate_disposal_code" ###
    # If result is a char *, need to deallocate the arg
    # now, because it wasn't done before.
    if self.type.is_string:
      if debug_disposal_code:
        print "CoerceFromPyTypeNode.generate_disposal_code:"
        print "...generating subexpr disposal code for", self
      #self.arg.generate_disposal_code(code)
      self.generate_subexpr_disposal_code(code)
  
  def generate_post_assignment_code(self, code):
    if debug_disposal_code:
      print "CoerceFromPyTypeNode.generate_post_assignment_code:"
      print "...generating disposal code for", self
    self.generate_disposal_code(code)


class CoerceToBooleanNode(CoercionNode):

  def __init__(self, arg, env):
    CoercionNode.__init__(self, arg)
    self.type = PyrexTypes.c_int_type
    if arg.type.is_pyobject:
      self.allocate_temp(env)
    else:
      self.result = "(%s != 0)" % arg.result
    arg.release_temp(env)
  
  def check_const(self):
    if not self.is_temp:
      self.arg.check_const()
    else:
      self.not_const()
  
  def generate_evaluation_code(self, code):
    #print "CoerceToBooleanNode.generate_evaluation_code:" ###
    #print "...generating evaluation code for arg (%s)" % self.arg ###
    self.arg.generate_evaluation_code(code)
    if self.arg.type.is_pyobject:
      code.putln(
        "%s = PyObject_IsTrue(%s); if (%s < 0) %s" % (
          self.result, 
          self.arg.result, 
          self.result,
          code.error_goto(self.pos)))
    if debug_disposal_code:
      print "CoerceToBooleanNode.generate_evaluation_code:"
      print "...generating subexpr disposal code for", self
    #self.arg.generate_disposal_code(code)
    self.generate_subexpr_disposal_code(code)


class CoerceToCharNode(CoercionNode):
  #  Convert char array or pointer to character.
  
  def __init__(self, arg):
    CoercionNode.__init__(self, arg)
    self.type = PyrexTypes.c_char_type
    self.result = "(%s[0])" % arg.result
  
  def generate_evaluation_code(self, code):
    self.arg.generate_evaluation_code(code)


class CoerceToTempNode(CoercionNode):

  def __init__(self, arg, env):
    CoercionNode.__init__(self, arg)
    self.type = self.arg.type
    self.allocate_temp(env)
  
  def generate_evaluation_code(self, code):
    self.arg.generate_evaluation_code(code)
    code.putln("%s = %s;" % (
      self.result, self.arg.result))
    if self.type.is_pyobject and not self.arg.is_temp:
      code.put_incref(self.result)


class CloneNode(CoercionNode):
  #  This node is employed when the result of another node needs
  #  to be used multiple times. The argument node's result must
  #  be in a temporary. The owner of the argument node is
  #  responsible for generating evaluation and disposal code
  #  for it.
  
  def __init__(self, arg):
    CoercionNode.__init__(self, arg)
    self.type = arg.type
    self.result = arg.result
  
  def generate_evaluation_code(self, code):
    return
  
  def generate_subexpr_disposal_code(self, code):
    return
  
#------------------------------------------------------------------------------------
#
#  Runtime support code
#
#------------------------------------------------------------------------------------

get_name_utility_code = \
"""
static PyObject *__Pyx_GetName(PyObject *dict, char *name) {
  PyObject *result;
  result = PyObject_GetAttrString(dict, name);
  if (!result)
    PyErr_SetString(PyExc_NameError, name);
  return result;
}
"""

#------------------------------------------------------------------------------------

import_utility_code = \
"""
static PyObject *__Pyx_Import(PyObject *name, PyObject *from_list) {
  PyObject *__import__ = 0;
  PyObject *empty_list = 0;
  PyObject *module = 0;
  PyObject *global_dict = 0;
  PyObject *empty_dict = 0;
  PyObject *list;
  __import__ = PyObject_GetAttrString(%(BUILTINS)s, "__import__");
  if (!__import__)
    goto bad;
  if (from_list)
    list = from_list;
  else {
    empty_list = PyList_New(0);
    if (!empty_list)
      goto bad;
    list = empty_list;
  }
  global_dict = PyModule_GetDict(%(GLOBALS)s);
  if (!global_dict)
    goto bad;
  empty_dict = PyDict_New();
  if (!empty_dict)
    goto bad;
  module = PyObject_CallFunction(__import__, "OOOO",
    name, global_dict, empty_dict, list);
bad:
  Py_XDECREF(empty_list);
  Py_XDECREF(__import__);
  Py_XDECREF(empty_dict);
  return module;
}
""" % {
  "BUILTINS": Naming.builtins_cname,
  "GLOBALS":  Naming.module_cname,
}

#------------------------------------------------------------------------------------

get_exception_utility_code = \
"""
static PyObject *__Pyx_GetExcValue(void) {
  PyObject *type = 0, *value = 0, *tb = 0;
  PyObject *result = 0;
  PyThreadState *tstate = PyThreadState_Get();
  PyErr_Fetch(&type, &value, &tb);
  PyErr_NormalizeException(&type, &value, &tb);
  if (PyErr_Occurred())
    goto bad;
  if (!value) {
    value = Py_None;
    Py_INCREF(value);
  }
  Py_XDECREF(tstate->exc_type);
  Py_XDECREF(tstate->exc_value);
  Py_XDECREF(tstate->exc_traceback);
  tstate->exc_type = type;
  tstate->exc_value = value;
  tstate->exc_traceback = tb;
  result = value;
  Py_XINCREF(result);
  type = 0;
  value = 0;
  tb = 0;
bad:
  Py_XDECREF(type);
  Py_XDECREF(value);
  Py_XDECREF(tb);
  return result;
}
"""

#------------------------------------------------------------------------------------

unpacking_utility_code = \
"""
static void __Pyx_UnpackError(void) {
  PyErr_SetString(PyExc_ValueError, "unpack sequence of wrong size");
}

static PyObject *__Pyx_UnpackItem(PyObject *seq, int i) {
  PyObject *item;
  if (!(item = PySequence_GetItem(seq, i))) {
    if (PyErr_ExceptionMatches(PyExc_IndexError))
      __Pyx_UnpackError();
  }
  return item;
}

static int __Pyx_EndUnpack(PyObject *seq, int i) {
  PyObject *item;
  if (item = PySequence_GetItem(seq, i)) {
    Py_DECREF(item);
    __Pyx_UnpackError();
    return -1;
  }
  PyErr_Clear();
  return 0;
}
"""

#------------------------------------------------------------------------------------

type_test_utility_code = \
"""
static int __Pyx_TypeTest(PyObject *obj, PyTypeObject *type) {
  if (PyObject_TypeCheck(obj, type))
    return 1;
  PyErr_Format(PyExc_TypeError, "Cannot convert %s to %s",
    obj->ob_type->tp_name, type->tp_name);
  return 0;
}
"""

#------------------------------------------------------------------------------------


  
