#
#   Pyrex Parser
#

import re
from string import join, replace
from types import ListType, TupleType
import Nodes
import ExprNodes
from Errors import error, InternalError

def p_ident(s, message = "Expected an identifier"):
  if s.sy == 'IDENT':
    name = s.systring
    s.next()
    return name
  else:
    s.error(message)

def p_ident_list(s):
  names = []
  while s.sy == 'IDENT':
    names.append(s.systring)
    s.next()
    if s.sy <> ',':
      break
    s.next()
  return names

#------------------------------------------
#
#   Expressions
#
#------------------------------------------

def p_binop_expr(s, ops, p_sub_expr):
  #print "p_binop_expr:", ops, p_sub_expr ###
  n1 = p_sub_expr(s)
  #print "p_binop_expr(%s):" % p_sub_expr, s.sy ###
  while s.sy in ops:
    op = s.sy
    pos = s.position()
    s.next()
    n2 = p_sub_expr(s)
    n1 = ExprNodes.binop_node(pos, op, n1, n2)
  return n1

#test: and_test ('or' and_test)* | lambdef

def p_simple_expr(s):
  #return p_binop_expr(s, ('or',), p_and_test)
  return p_rassoc_binop_expr(s, ('or',), p_and_test)

def p_rassoc_binop_expr(s, ops, p_subexpr):
  n1 = p_subexpr(s)
  if s.sy in ops:
    pos = s.position()
    op = s.sy
    s.next()
    n2 = p_rassoc_binop_expr(s, ops, p_subexpr)
    n1 = ExprNodes.binop_node(pos, op, n1, n2)
  return n1

#and_test: not_test ('and' not_test)*

def p_and_test(s):
  #return p_binop_expr(s, ('and',), p_not_test)
  return p_rassoc_binop_expr(s, ('and',), p_not_test)

#not_test: 'not' not_test | comparison

def p_not_test(s):
  if s.sy == 'not':
    pos = s.position()
    s.next()
    return ExprNodes.NotNode(pos, operand = p_not_test(s))
  else:
    return p_comparison(s)

#comparison: expr (comp_op expr)*
#comp_op: '<'|'>'|'=='|'>='|'<='|'<>'|'!='|'in'|'not' 'in'|'is'|'is' 'not'

def p_comparison(s):
  n1 = p_bit_expr(s)
  if s.sy in comparison_ops:
    pos = s.position()
    op = p_cmp_op(s)
    n2 = p_bit_expr(s)
    n1 = ExprNodes.PrimaryCmpNode(pos, 
      operator = op, operand1 = n1, operand2 = n2)
    if s.sy in comparison_ops:
      n1.cascade = p_cascaded_cmp(s)
  return n1

def p_cascaded_cmp(s):
  pos = s.position()
  op = p_cmp_op(s)
  n2 = p_bit_expr(s)
  result = ExprNodes.CascadedCmpNode(pos, 
    operator = op, operand2 = n2)
  if s.sy in comparison_ops:
    result.cascade = p_cascaded_cmp(s)
  return result

def p_cmp_op(s):
  if s.sy == 'not':
    s.next()
    s.expect('in')
    op = 'not_in'
  elif s.sy == 'is':
    s.next()
    if s.sy == 'not':
      s.next()
      op = 'is_not'
    else:
      op = 'is'
  else:
    op = s.sy
    s.next()
  if op == '<>':
    op = '!='
  return op
  
comparison_ops = (
  '<', '>', '==', '>=', '<=', '<>', '!=', 
  'in', 'is', 'not'
)

#expr: xor_expr ('|' xor_expr)*

def p_bit_expr(s):
  return p_binop_expr(s, ('|',), p_xor_expr)

#xor_expr: and_expr ('^' and_expr)*

def p_xor_expr(s):
  return p_binop_expr(s, ('^',), p_and_expr)

#and_expr: shift_expr ('&' shift_expr)*

def p_and_expr(s):
  return p_binop_expr(s, ('&',), p_shift_expr)

#shift_expr: arith_expr (('<<'|'>>') arith_expr)*

def p_shift_expr(s):
  return p_binop_expr(s, ('<<', '>>'), p_arith_expr)

#arith_expr: term (('+'|'-') term)*

def p_arith_expr(s):
  return p_binop_expr(s, ('+', '-'), p_term)

#term: factor (('*'|'/'|'%') factor)*

def p_term(s):
  return p_binop_expr(s, ('*', '/', '%'), p_factor)

#factor: ('+'|'-'|'~'|'&'|typecast|sizeof) factor | power

def p_factor(s):
  sy = s.sy
  if sy in ('+', '-', '~'):
    op = s.sy
    pos = s.position()
    s.next()
    return ExprNodes.unop_node(pos, op, p_factor(s))
  elif sy == '&':
    pos = s.position()
    s.next()
    arg = p_factor(s)
    return ExprNodes.AmpersandNode(pos, operand = arg)
  elif sy == "<":
    return p_typecast(s)
  elif sy == 'IDENT' and s.systring == "sizeof":
    return p_sizeof(s)
  else:
    return p_power(s)

def p_typecast(s):
  # s.sy == "<"
  pos = s.position()
  s.next()
  base_type = p_c_base_type(s)
  declarator = p_c_declarator(s, empty = 1)
  s.expect(">")
  operand = p_factor(s)
  return ExprNodes.TypecastNode(pos, 
    base_type = base_type, 
    declarator = declarator,
    operand = operand)

def p_sizeof(s):
  # s.sy == ident "sizeof"
  pos = s.position()
  s.next()
  s.expect('(')
  if looking_at_type(s):
    base_type = p_c_base_type(s)
    declarator = p_c_declarator(s, empty = 1)
    node = ExprNodes.SizeofTypeNode(pos, 
      base_type = base_type, declarator = declarator)
  else:
    operand = p_simple_expr(s)
    node = ExprNodes.SizeofVarNode(pos, operand = operand)
  s.expect(')')
  return node

#power: atom trailer* ('**' factor)*

def p_power(s):
  n1 = p_atom(s)
  while s.sy in ('(', '[', '.'):
    n1 = p_trailer(s, n1)
  if s.sy == '**':
    pos = s.position()
    s.next()
    n2 = p_factor(s)
    n1 = ExprNodes.binop_node(pos, '**', n1, n2)
  return n1

#trailer: '(' [arglist] ')' | '[' subscriptlist ']' | '.' NAME

def p_trailer(s, node1):
  pos = s.position()
  if s.sy == '(':
    return p_call(s, node1)
  elif s.sy == '[':
    return p_index(s, node1)
  else: # s.sy == '.'
    s.next()
    name = p_ident(s)
    return ExprNodes.AttributeNode(pos, 
      obj = node1, attribute = name)

# arglist:  argument (',' argument)* [',']
# argument: [test '='] test       # Really [keyword '='] test

def p_call(s, function):
  # s.sy == '('
  pos = s.position()
  s.next()
  positional_args = []
  keyword_args = []
  star_arg = None
  starstar_arg = None
  while s.sy not in ('*', '**', ')'):
    arg = p_simple_expr(s)
    if s.sy == '=':
      s.next()
      if not arg.is_name:
        s.error("Expected an identifier before '='",
          pos = arg.pos)
      keyword = ExprNodes.StringNode(arg.pos, 
        value = arg.name)
      arg = p_simple_expr(s)
      keyword_args.append((keyword, arg))
    else:
      if keyword_args:
        s.error("Non-keyword arg following keyword arg",
          pos = arg.pos)
      positional_args.append(arg)
    if s.sy <> ',':
      break
    s.next()
  if s.sy == '*':
    s.next()
    star_arg = p_simple_expr(s)
    if s.sy == ',':
      s.next()
  if s.sy == '**':
    s.next()
    starstar_arg = p_simple_expr(s)
    if s.sy == ',':
      s.next()
  s.expect(')')
  if not (keyword_args or star_arg or starstar_arg):
    return ExprNodes.SimpleCallNode(pos,
      function = function,
      args = positional_args)
  else:
    arg_tuple = None
    keyword_dict = None
    if positional_args or not star_arg:
      arg_tuple = ExprNodes.TupleNode(pos, 
        args = positional_args)
    if star_arg:
      star_arg_tuple = ExprNodes.AsTupleNode(pos, arg = star_arg)
      if arg_tuple:
        arg_tuple = ExprNodes.binop_node(pos, 
          operator = '+', operand1 = arg_tuple,
          operand2 = star_arg_tuple)
      else:
        arg_tuple = star_arg_tuple
    if keyword_args:
      keyword_dict = ExprNodes.DictNode(pos,
        key_value_pairs = keyword_args)
    return ExprNodes.GeneralCallNode(pos, 
      function = function,
      positional_args = arg_tuple,
      keyword_args = keyword_dict,
      starstar_arg = starstar_arg)

#lambdef: 'lambda' [varargslist] ':' test

#subscriptlist: subscript (',' subscript)* [',']

def p_index(s, base):
  # s.sy == '['
  pos = s.position()
  s.next()
  subscripts = p_subscript_list(s)
  if len(subscripts) == 1 and len(subscripts[0]) == 2:
    start, stop = subscripts[0]
    result = ExprNodes.SliceIndexNode(pos, 
      base = base, start = start, stop = stop)
  else:
    indexes = make_slice_nodes(pos, subscripts)
    if len(indexes) == 1:
      index = indexes[0]
    else:
      index = ExprNodes.TupleNode(pos, args = indexes)
    result = ExprNodes.IndexNode(pos,
      base = base, index = index)
  s.expect(']')
  return result

def p_subscript_list(s):
  items = [p_subscript(s)]
  while s.sy == ',':
    s.next()
    if s.sy == ']':
      break
    items.append(p_subscript(s))
  return items

#subscript: '.' '.' '.' | test | [test] ':' [test] [':' [test]]

def p_subscript(s):
  # Parse a subscript and return a list of
  # 1, 2 or 3 ExprNodes, depending on how
  # many slice elements were encountered.
  pos = s.position()
  if s.sy == '.':
    expect_ellipsis(s)
    return [ExprNodes.EllipsisNode(pos)]
  else:
    start = p_slice_element(s, (':',))
    if s.sy <> ':':
      return [start]
    s.next()
    stop = p_slice_element(s, (':', ',', ']'))
    if s.sy <> ':':
      return [start, stop]
    s.next()
    step = p_slice_element(s, (':', ',', ']'))
    return [start, stop, step]

def p_slice_element(s, follow_set):
  # Simple expression which may be missing iff
  # it is followed by something in follow_set.
  if s.sy not in follow_set:
    return p_simple_expr(s)
  else:
    return None

def expect_ellipsis(s):
  s.expect('.')
  s.expect('.')
  s.expect('.')

def make_slice_nodes(pos, subscripts):
  # Convert a list of subscripts as returned
  # by p_subscript_list into a list of ExprNodes,
  # creating SliceNodes for elements with 2 or
  # more components.
  result = []
  for subscript in subscripts:
    if len(subscript) == 1:
      result.append(subscript[0])
    else:
      result.append(make_slice_node(pos, *subscript))
  return result

def make_slice_node(pos, start, stop = None, step = None):
  if not start:
    start = ExprNodes.NoneNode(pos)
  if not stop:
    stop = ExprNodes.NoneNode(pos)
  if not step:
    step = ExprNodes.NoneNode(pos)
  return ExprNodes.SliceNode(pos,
    start = start, stop = stop, step = step)

#atom: '(' [testlist] ')' | '[' [listmaker] ']' | '{' [dictmaker] '}' | '`' testlist '`' | NAME | NUMBER | STRING+

def p_atom(s):
  pos = s.position()
  sy = s.sy
  if sy == '(':
    s.next()
    if s.sy == ')':
      result = ExprNodes.TupleNode(pos, args = [])
    else:
      result = p_expr(s)
    s.expect(')')
    return result
  elif sy == '[':
    return p_list_maker(s)
  elif sy == '{':
    return p_dict_maker(s)
  elif sy == '`':
    return p_backquote_expr(s)
  elif sy == 'INT':
    digits = s.systring
    if digits[:2] == "0x":
      value = int(digits[2:], 16)
    elif digits[:1] == "0":
      value = int(digits, 8)
    else:
      value = int(s.systring)
    s.next()
    return ExprNodes.IntNode(pos, value = value)
  elif sy == 'FLOAT':
    value = float(s.systring)
    s.next()
    return ExprNodes.FloatNode(pos, value = value)
  elif sy == 'IMAG':
    value = float(s.systring[:-1])
    s.next()
    return ExprNodes.ImagNode(pos, value = value)
  elif sy == 'STRING' or sy == 'BEGIN_STRING':
    value = p_cat_string_literal(s)
    return ExprNodes.StringNode(pos, value = value)
  elif sy == 'IDENT':
    name = s.systring
    s.next()
    return ExprNodes.NameNode(pos, name=name)
  elif sy == 'NULL':
    s.next()
    return ExprNodes.NullNode(pos)
  else:
    s.error("Expected an identifier or literal")

def p_cat_string_literal(s):
  strings = []
  while s.sy == 'STRING' or s.sy == 'BEGIN_STRING':
    strings.append(p_string_literal(s))
  return ''.join(strings)

def p_opt_string_literal(s):
  if s.sy == 'STRING' or s.sy == 'BEGIN_STRING':
    return p_string_literal(s)
  else:
    return None

def p_string_literal(s):
  if s.sy == 'STRING':
    value = unquote(s.systring)
    s.next()
    return value
  # s.sy == 'BEGIN_STRING'
  pos = s.position()
  is_raw = s.systring[:1].lower() == "r"
  chars = []
  while 1:
    s.next()
    sy = s.sy
    #print "p_string_literal: sy =", sy, repr(s.systring) ###
    if sy == 'CHARS':
      systr = s.systring
      if len(systr) == 1 and systr in "'\"\n":
        chars.append('\\')
      chars.append(systr)
    elif sy == 'ESCAPE':
      systr = s.systring
      if is_raw:
        if systr == '\\\n':
          chars.append(r'\\\n')
        else:
          chars.append('\\' + systr)
      else:
        c = systr[1]
        if c in "'\"\\abfnrtv01234567":
          chars.append(systr)
        elif c == 'x':
          chars.append('\\x0' + systr[2:])
        elif c == '\n':
          pass
        else:
          chars.append(systr[1:])
    elif sy == 'NEWLINE':
      chars.append(r'\n')
    elif sy == 'END_STRING':
      break
    elif sy == 'EOF':
      s.error("Unclosed string literal", pos = pos)
    else:
      s.error(
        "Unexpected token %r:%r in string literal" %
          (sy, s.systring))
  s.next()
  value = join(chars, '')
  #print "p_string_literal: value =", repr(value) ###
  return value

def unquote(s):
  is_raw = 0
  if s[:1].lower() == "r":
    is_raw = 1
    s = s[1:]
  q = s[:3]
  if q == '"""' or q == "'''":
    s = s[3:-3]
  else:
    s = s[1:-1]
  if is_raw:
    s = s.replace('\\', '\\\\')
    s = s.replace('\n', '\\\n')
  else:
    # Split into double quotes, newlines, escape sequences 
    # and spans of regular chars
    l1 = re.split(r'((?:\\[0-7]{1,3})|(?:\\x[0-9A-Fa-f]{2})|(?:\\.)|(?:\\\n)|(?:\n)|")', s)
    print "unquote: l1 =", l1 ###
    l2 = []
    for item in l1:
      if item == '"' or item == '\n':
        l2.append('\\' + item)
      elif item == '\\\n':
        pass
      elif item[:1] == '\\':
        if len(item) == 2:
          if item[1] in '"\\abfnrtv':
            l2.append(item)
          else:
            l2.append(item[1])
        elif item[1:2] == 'x':
          l2.append('\\x0' + item[2:])
        else:
          # octal escape
          l2.append(item)
      else:
        l2.append(item)
    s = "".join(l2)
  return s
    
def p_list_maker(s):
  # s.sy == '['
  pos = s.position()
  s.next()
  exprs = p_simple_expr_list(s)
  s.expect(']')
  return ExprNodes.ListNode(pos, args = exprs)

#dictmaker: test ':' test (',' test ':' test)* [',']

def p_dict_maker(s):
  # s.sy == '{'
  pos = s.position()
  s.next()
  items = []
  while s.sy <> '}':
    key = p_simple_expr(s)
    s.expect(':')
    value = p_simple_expr(s)
    items.append((key, value))
    if s.sy <> ',':
      break
    s.next()
  s.expect('}')
  return ExprNodes.DictNode(pos, key_value_pairs = items)

def p_backquote_expr(s):
  # s.sy == '`'
  pos = s.position()
  s.next()
  arg = p_expr(s)
  s.expect('`')
  return ExprNodes.BackquoteNode(pos, arg = arg)

#testlist: test (',' test)* [',']

def p_simple_expr_list(s):
  exprs = []
  while s.sy not in expr_terminators:
    exprs.append(p_simple_expr(s))
    if s.sy <> ',':
      break
    s.next()
  return exprs

def p_expr(s):
  pos = s.position()
  expr = p_simple_expr(s)
  if s.sy == ',':
    s.next()
    exprs = [expr] + p_simple_expr_list(s)
    return ExprNodes.TupleNode(pos, args = exprs)
  else:
    return expr

expr_terminators = (')', ']', '}', ':', '=', 'EOL')

#-------------------------------------------------------
#
#   Statements
#
#-------------------------------------------------------

def p_global_statement(s):
  # assume s.sy == 'global'
  pos = s.position()
  s.next()
  names = p_ident_list(s)
  return Nodes.GlobalNode(pos, names = names)

def p_expression_or_assignment(s):
  expr_list = [p_expr(s)]
  while s.sy == '=':
    s.next()
    expr_list.append(p_expr(s))
  if len(expr_list) == 1:
    expr = expr_list[0]
    return Nodes.ExprStatNode(expr.pos, expr = expr)
  else:
    expr_list_list = []
    flatten_parallel_assignments(expr_list, expr_list_list)
    nodes = []
    for expr_list in expr_list_list:
      lhs_list = expr_list[:-1]
      rhs = expr_list[-1]
      if len(lhs_list) == 1:
        node = Nodes.SingleAssignmentNode(rhs.pos, 
          lhs = lhs_list[0], rhs = rhs)
      else:
        node = Nodes.CascadedAssignmentNode(rhs.pos,
          lhs_list = lhs_list, rhs = rhs)
      nodes.append(node)
    if len(nodes) == 1:
      return nodes[0]
    else:
      return Nodes.StatListNode(nodes[0].pos, stats = nodes)

def flatten_parallel_assignments(input, output):
  #  The input is a list of expression nodes,
  #  representing the LHSs and RHS of one
  #  assignment statement. If they are all
  #  sequence constructors with the same number
  #  of arguments, rearranges them into a list
  #  of equivalent assignments between the
  #  individual elements. This transformation is
  #  applied recursively.
  size = find_parallel_assignment_size(input)
  if size >= 0:
    for i in range(size):
      new_exprs = [expr.args[i] for expr in input]
      flatten_parallel_assignments(new_exprs, output)
  else:
    output.append(input)

def find_parallel_assignment_size(input):
  #  The input is a list of expression nodes. If 
  #  they are all sequence constructors with the same number
  #  of arguments, return that number, else return -1.
  #  Produces an error message if they are all sequence
  #  constructors but not all the same size.
  for expr in input:
    if not expr.is_sequence_constructor:
      return -1
  rhs = input[-1]
  rhs_size = len(rhs.args)
  for lhs in input[:-1]:
    lhs_size = len(lhs.args)
    if lhs_size <> rhs_size:
      error(lhs.pos, "Unpacking sequence of wrong size (expected %d, got %d)"
        % (lhs_size, rhs_size))
      return -1
  return rhs_size

def p_print_statement(s):
  # s.sy == 'print'
  pos = s.position()
  s.next()
  if s.sy == '>>':
    s.error("'print >>' not yet implemented")
  args = []
  ewc = 0
  if s.sy <> 'NEWLINE':
    args.append(p_simple_expr(s))
    while s.sy == ',':
      s.next()
      if s.sy == 'NEWLINE':
        ewc = 1
        break
      args.append(p_simple_expr(s))
  return Nodes.PrintStatNode(pos, 
    args = args, ends_with_comma = ewc)

def p_del_statement(s):
  # s.sy == 'del'
  pos = s.position()
  s.next()
  args = p_simple_expr_list(s)
  return Nodes.DelStatNode(pos, args = args)

def p_pass_statement(s):
  # s.sy == 'pass'
  pos = s.position()
  s.next()
  return Nodes.PassStatNode(pos)

def p_break_statement(s):
  # s.sy == 'break'
  pos = s.position()
  s.next()
  return Nodes.BreakStatNode(pos)

def p_continue_statement(s):
  # s.sy == 'continue'
  pos = s.position()
  s.next()
  return Nodes.ContinueStatNode(pos)

def p_return_statement(s):
  # s.sy == 'return'
  pos = s.position()
  s.next()
  if s.sy not in statement_terminators:
    value = p_expr(s)
  else:
    value = None
  return Nodes.ReturnStatNode(pos, value = value)

def p_raise_statement(s):
  # s.sy == 'raise'
  pos = s.position()
  s.next()
  exc_type = None
  exc_value = None
  exc_tb = None
  if s.sy not in statement_terminators:
    exc_type = p_simple_expr(s)
    if s.sy == ',':
      s.next()
      exc_value = p_simple_expr(s)
      if s.sy == ',':
        s.next()
        exc_tb = p_simple_expr(s)
  return Nodes.RaiseStatNode(pos, 
    exc_type = exc_type,
    exc_value = exc_value,
    exc_tb = exc_tb)

def p_import_statement(s):
  # s.sy == 'import'
  pos = s.position()
  s.next()
  items = [p_dotted_name(s, as_allowed = 1)]
  while s.sy == ',':
    s.next()
    items.append(p_dotted_name(s, as_allowed = 1))
  stats = []
  for pos, target_name, dotted_name, as_name in items:
    stat = Nodes.SingleAssignmentNode(pos,
      lhs = ExprNodes.NameNode(pos, 
        name = as_name or target_name),
      rhs = ExprNodes.ImportNode(pos, 
        module_name = ExprNodes.StringNode(pos,
          value = dotted_name),
        name_list = None))
    stats.append(stat)
  return Nodes.StatListNode(pos, stats = stats)

def p_from_import_statement(s):
  # s.sy == 'from'
  pos = s.position()
  s.next()
  (dotted_name_pos, _, dotted_name, _) = \
    p_dotted_name(s, as_allowed = 0)
  s.expect('import')
  if s.sy == '*':
    s.error("'import *' not supported")
  imported_names = [p_imported_name(s)]
  while s.sy == ',':
    s.next()
    imported_names.append(p_imported_name(s))
  imported_name_strings = []
  items = []
  for (name_pos, name, as_name) in imported_names:
    imported_name_strings.append(
      ExprNodes.StringNode(name_pos, value = name))
    items.append(
      (name,
       ExprNodes.NameNode(name_pos, 
        name = as_name or name)))
  import_list = ExprNodes.ListNode(
    imported_names[0][0], args = imported_name_strings)
  return Nodes.FromImportStatNode(pos,
    module = ExprNodes.ImportNode(dotted_name_pos,
      module_name = ExprNodes.StringNode(dotted_name_pos,
        value = dotted_name),
      name_list = import_list),
    items = items)

def p_imported_name(s):
  pos = s.position()
  name = p_ident(s)
  as_name = p_as_name(s)
  return (pos, name, as_name)

def p_dotted_name(s, as_allowed):
  pos = s.position()
  target_name = p_ident(s)
  as_name = None
  names = [target_name]
  while s.sy == '.':
    s.next()
    names.append(p_ident(s))
  if as_allowed:
    as_name = p_as_name(s)
  else:
    as_name = None
  return (pos, target_name, join(names, "."), as_name)

def p_as_name(s):
  if s.sy == 'IDENT' and s.systring == 'as':
    s.next()
    return p_ident(s)
  else:
    return None

def p_assert_statement(s):
  # s.sy == 'assert'
  pos = s.position()
  s.next()
  expr = p_expr(s)
  return Nodes.AssertStatNode(pos, expr = expr)

statement_terminators = (';', 'NEWLINE', 'EOF')

def p_simple_statement(s):
  if s.sy == 'global':
    node = p_global_statement(s)
  elif s.sy == 'print':
    node = p_print_statement(s)
  elif s.sy == 'del':
    node = p_del_statement(s)
  #elif s.sy == 'pass':
  #	node = p_pass_statement(s)
  elif s.sy == 'break':
    node = p_break_statement(s)
  elif s.sy == 'continue':
    node = p_continue_statement(s)
  elif s.sy == 'return':
    node = p_return_statement(s)
  elif s.sy == 'raise':
    node = p_raise_statement(s)
  elif s.sy == 'import':
    node = p_import_statement(s)
  elif s.sy == 'from':
    node = p_from_import_statement(s)
  elif s.sy == 'assert':
    node = p_assert_statement(s)
#	elif s.sy == 'xxx':
#		node = p_xxx_statement(s)
  else:
    node = p_expression_or_assignment(s)
  if s.sy == 'NEWLINE':
    s.next()
  else:
    s.error("Syntax error in statement")
  return node

def p_if_statement(s):
  # s.sy == 'if'
  pos = s.position()
  s.next()
  if_clauses = [p_if_clause(s)]
  while s.sy == 'elif':
    s.next()
    if_clauses.append(p_if_clause(s))
  else_clause = p_else_clause(s)
  return Nodes.IfStatNode(pos,
    if_clauses = if_clauses, else_clause = else_clause)

def p_if_clause(s):
  pos = s.position()
  test = p_simple_expr(s)
  body = p_suite(s)
  return Nodes.IfClauseNode(pos,
    condition = test, body = body)

def p_else_clause(s):
  if s.sy == 'else':
    s.next()
    return p_suite(s)
  else:
    return None

def p_while_statement(s):
  # s.sy == 'while'
  pos = s.position()
  s.next()
  test = p_simple_expr(s)
  body = p_suite(s)
  else_clause = p_else_clause(s)
  return Nodes.WhileStatNode(pos, 
    condition = test, body = body, 
    else_clause = else_clause)

def p_for_statement(s):
  # s.sy == 'for'
  pos = s.position()
  s.next()
  target = p_for_target(s)
  if s.sy == 'in':
    s.next()
    iterator = p_for_iterator(s)
    body = p_suite(s)
    else_clause = p_else_clause(s)
    return Nodes.ForInStatNode(pos, 
      target = target,
      iterator = iterator,
      body = body,
      else_clause = else_clause)
  elif s.sy == 'from':
    s.next()
    bound1 = p_bit_expr(s)
    rel1 = p_for_from_relation(s)
    name2_pos = s.position()
    name2 = p_ident(s)
    rel2_pos = s.position()
    rel2 = p_for_from_relation(s)
    bound2 = p_bit_expr(s)
    if not target.is_name:
      error(target.pos, 
        "Target of for-from statement must be a variable name")
    elif name2 <> target.name:
      error(name2_pos,
        "Variable name in for-from range does not match target")
    if rel1[0] <> rel2[0]:
      error(rel2_pos,
        "Relation directions in for-from do not match")
    body = p_suite(s)
    else_clause = p_else_clause(s)
    return Nodes.ForFromStatNode(pos,
      target = target,
      bound1 = bound1,
      relation1 = rel1,
      relation2 = rel2,
      bound2 = bound2,
      body = body,
      else_clause = else_clause)

def p_for_from_relation(s):
  if s.sy in inequality_relations:
    op = s.sy
    s.next()
    return op
  else:
    s.error("Expected one of '<', '<=', '>' '>='")

inequality_relations = ('<', '<=', '>', '>=')

def p_for_target(s):
  pos = s.position()
  expr = p_bit_expr(s)
  if s.sy == ',':
    s.next()
    exprs = [expr]
    while s.sy <> 'in':
      exprs.append(p_bit_expr(s))
      if s.sy <> ',':
        break
      s.next()
    return ExprNodes.TupleNode(pos, args = exprs)
  else:
    return expr

def p_for_iterator(s):
  pos = s.position()
  expr = p_expr(s)
  return ExprNodes.IteratorNode(pos, sequence = expr)

def p_try_statement(s):
  # s.sy == 'try'
  pos = s.position()
  s.next()
  body = p_suite(s)
  except_clauses = []
  if s.sy == 'except':
    while s.sy == 'except':
      except_clauses.append(p_except_clause(s))
    return Nodes.TryExceptStatNode(pos,
      body = body, except_clauses = except_clauses)
  elif s.sy == 'finally':
    s.next()
    finally_clause = p_suite(s)
    return Nodes.TryFinallyStatNode(pos,
      body = body, finally_clause = finally_clause)
  else:
    s.error("Expected 'except' or 'finally'")

def p_except_clause(s):
  # s.sy == 'except'
  pos = s.position()
  s.next()
  exc_type = None
  exc_value = None
  if s.sy <> ':':
    exc_type = p_simple_expr(s)
    if s.sy == ',':
      s.next()
      exc_value = p_simple_expr(s)
  body = p_suite(s)
  return Nodes.ExceptClauseNode(pos,
    pattern = exc_type, target = exc_value, body = body)

def p_statement(s, level, cdef_flag = 0, extern_flag = 0):
  if s.sy == 'ctypedef':
    if not (level == 'module'):
      s.error("ctypedef statement not allowed here")
    return p_ctypedef_statement(s, 
      extern_flag = extern_flag)
  if s.sy == 'cdef':
    cdef_flag = 1
    s.next()
  if cdef_flag:
    if not (level == 'module' or level == 'function'
        or level == 'c_class'):
      s.error('cdef statement not allowed here')
    return p_cdef_statement(s, level,
      extern_flag = extern_flag)
  elif s.sy == 'def':
    if not (level == 'module' or level == 'class'
        or level == 'c_class'):
      s.error('def statement not allowed here')
    return p_def_statement(s)
  elif s.sy == 'class':
    if level <> 'module':
      s.error("class definition not allowed here")
    return p_class_statement(s)
  elif s.sy == 'if':
    return p_if_statement(s)
  elif s.sy == 'while':
    return p_while_statement(s)
  elif s.sy == 'for':
    return p_for_statement(s)
  elif s.sy == 'try':
    return p_try_statement(s)
#	elif s.sy == 'xxx':
#		return p_xxx_statement(s)
  else:
    return p_simple_statement(s)

def p_statement_list(s, level,
    cdef_flag = 0, extern_flag = 0):
  pos = s.position()
  stats = []
  while s.sy <> 'EOF' and s.sy <> 'DEDENT':
    if s.sy == 'pass':
      s.next()
      s.expect('NEWLINE', "Syntax error in statement")
    else:
      stats.append(p_statement(s, level,
        cdef_flag = cdef_flag, extern_flag = extern_flag))
  return Nodes.StatListNode(pos, stats = stats)

def p_suite(s, level = 'other', cdef_flag = 0,
    extern_flag = 0, with_doc = 0):
  pos = s.position()
  s.expect(':')
  doc = None
  stmts = []
  if s.sy == 'NEWLINE':
    s.next()
    s.expect_indent()
    if with_doc:
      doc = p_doc_string(s)
    body = p_statement_list(s, 
      level = level,
      cdef_flag = cdef_flag, 
      extern_flag = extern_flag)
    s.expect_dedent()
  else:
    body = p_simple_statement(s)
  if with_doc:
    return doc, body
  else:
    return body

def p_c_base_type(s):
  if s.sy == '(':
    return p_c_complex_base_type(s)
  else:
    return p_c_simple_base_type(s)

def p_c_complex_base_type(s):
  # s.sy == '('
  pos = s.position()
  s.next()
  base_type = p_c_base_type(s)
  declarator = p_c_declarator(s, empty = 1)
  s.expect(')')
  return Nodes.CComplexBaseTypeNode(pos, 
    base_type = base_type, declarator = declarator)

def p_c_simple_base_type(s):
  is_basic = 0
  signed = 1
  longness = 0
  pos = s.position()
  if s.sy == 'IDENT' and s.systring in s.type_names:
    #print "p_c_simple_base_type:", pos ###
    #print "...absorbing:", s.sy, s.systring ###
    name = s.systring
    s.next()
  elif s.sy == 'IDENT' and s.systring in base_type_start_words:
    is_basic = 1
    signed = p_signed_or_unsigned(s)
    longness = p_short_or_long(s)
    if s.sy == 'IDENT' and s.systring in basic_c_type_names:
      name = s.systring
      s.next()
    else:
      name = 'int'
  else:
    name = "object"
  return Nodes.CSimpleBaseTypeNode(pos, name = name,
    is_basic_c_type = is_basic, signed = signed,
    longness = longness)

def looking_at_type(s):
  return s.sy == 'IDENT' \
    and (s.systring in s.type_names 
      or s.systring in base_type_start_words)

base_type_start_words = (
  "char", "short", "int", "long", "float", "double",
  "void", "signed", "unsigned"
)

basic_c_type_names = (
  "void", "char", "int", "float", "double"
)

def p_signed_or_unsigned(s):
  signed = 1
  if s.sy == 'IDENT':
    if s.systring == 'signed':
      s.next()
    elif s.systring == 'unsigned':
      signed = 0
      s.next()
  return signed

def p_short_or_long(s):
  longness = 0
  if s.sy == 'IDENT' and s.systring == 'short':
    longness = -1
    s.next()
  else:
    while s.sy == 'IDENT' and s.systring == 'long':
      longness += 1
      s.next()
  return longness

def p_c_declarator(s, empty = 0, is_type = 0):
  # If empty is true, the declarator must be
  # empty, otherwise we don't care.
  pos = s.position()
  if s.sy == '*':
    s.next()
    base = p_c_declarator(s, empty, is_type)
    result = Nodes.CPtrDeclaratorNode(pos, 
      base = base)
  elif s.sy == '**':
    s.next()
    base = p_c_declarator(s, empty, is_type)
    result = Nodes.CPtrDeclaratorNode(pos,
      base = Nodes.CPtrDeclaratorNode(pos,
        base = base))
  else:
    if s.sy == '(':
      s.next()
      result = p_c_declarator(s, empty, is_type)
      s.expect(')')
    else:
      if s.sy == 'IDENT':
        name = s.systring
        if is_type:
          s.type_names.append(name)
        s.next()
        if empty:
          error(s.position(), "Declarator should be empty")
      else:
        name = ""
      result = Nodes.CNameDeclaratorNode(pos, name = name)
    while s.sy in ('[', '('):
      if s.sy == '[':
        s.next()
        if s.sy <> ']':
          dim = p_expr(s)
        else:
          dim = None
        s.expect(']')
        result = Nodes.CArrayDeclaratorNode(pos, 
          base = result, dimension = dim)
      else: # sy == '('
        s.next()
        args = p_c_arg_list(s)
        ellipsis = p_optional_ellipsis(s)
        s.expect(')')
        exc_val, exc_check = p_exception_value_clause(s)
        result = Nodes.CFuncDeclaratorNode(pos, 
          base = result, args = args, has_varargs = ellipsis,
          exception_value = exc_val, exception_check = exc_check)
  return result

def p_exception_value_clause(s):
  exc_val = None
  exc_check = 0
  if s.sy == 'except':
    s.next()
    if s.sy == '*':
      exc_check = 1
      s.next()
    else:
      if s.sy == '?':
        exc_check = 1
        s.next()
      exc_val = p_exception_value(s)
  return exc_val, exc_check

def p_exception_value(s):
  sign = ""
  if s.sy == "-":
    sign = "-"
    s.next()
  if s.sy in ('INT', 'FLOAT', 'NULL'):
    s.systring = sign + s.systring
    return p_atom(s)
  else:
    s.error("Exception value must be an int or float literal or NULL")

c_arg_list_terminators = ('*', '**', '.', ')')
c_arg_list_trailers = ('.', '*', '**')

def p_c_arg_list(s):
  args = []
  if s.sy not in c_arg_list_terminators:
    args.append(p_c_arg_decl(s))
    while s.sy == ',':
      s.next()
      if s.sy in c_arg_list_trailers:
        break
      args.append(p_c_arg_decl(s))
  return args

def p_optional_ellipsis(s):
  if s.sy == '.':
    expect_ellipsis(s)
    return 1
  else:
    return 0

def p_c_arg_decl(s):
  pos = s.position()
  default = None
  base_type = p_c_base_type(s)
  declarator = p_c_declarator(s)
  if s.sy == '=':
    s.next()
    default = p_simple_expr(s)
  return Nodes.CArgDeclNode(pos,
    base_type = base_type,
    declarator = declarator,
    default = default)

def p_cdef_statement(s, level, extern_flag = 0):
  # s.sy == 'cdef'
  pos = s.position()
  if s.sy == 'cdef':
    s.next()
  if s.sy == 'IDENT' and s.systring == "extern":
    extern_flag = 1
    s.next()
  if extern_flag:
    if not (level == 'module'):
      s.error("'extern' declaration not allowed here")
    if s.sy == 'from' or s.sy == ':':
      return p_cdef_extern_block(s, pos)
  if s.sy == 'class':
    if not (level == 'module'):
      s.error("Extension type definition not allowed here")
    return p_c_class_definition(s, pos, 
      extern_flag = extern_flag)
  elif s.sy == 'IDENT' and s.systring in struct_union_or_enum:
    if not (level == 'module'):
      s.error("C struct/union/enum definition not allowed here")
    if s.systring == "enum":
      return p_c_enum_definition(s, pos)
    else:
      return p_c_struct_or_union_definition(s, pos)
  else:
    return p_c_func_or_var_declaration(s, pos, extern_flag)

def p_cdef_extern_block(s, pos):
  include_file = None
  s.expect('from')
  if s.sy == '*':
    s.next()
  else:
    include_file = p_string_literal(s)
  body = p_suite(s, level = 'module',
    cdef_flag = 1, extern_flag = 1)
  return Nodes.CDefExternNode(pos,
    include_file = include_file,
    body = body)

struct_union_or_enum = (
  "struct", "union", "enum"
)

def p_c_enum_definition(s, pos, typedef_flag = 0):
  # s.sy == ident 'enum'
  s.next()
  if s.sy == 'IDENT':
    name = s.systring
    s.next()
    s.type_names.append(name)
  else:
    name = None
  items = None
  s.expect(':')
  s.expect('NEWLINE')
  s.expect_indent()
  items = []
  while s.sy <> 'DEDENT':
    p_c_enum_line(s, items)
  s.expect_dedent()
  return Nodes.CEnumDefNode(pos, name = name, 
    items = items, typedef_flag = typedef_flag)

def p_c_enum_line(s, items):
  p_c_enum_item(s, items)
  while s.sy == ',':
    s.next()
    if s.sy == 'NEWLINE':
      break
    p_c_enum_item(s, items)
  s.expect('NEWLINE')

def p_c_enum_item(s, items):
  pos = s.position()
  name = p_ident(s)
  value = None
  if s.sy == '=':
    s.next()
    value = p_simple_expr(s)
  items.append(Nodes.CEnumDefItemNode(pos, 
    name = name, value = value))

def p_c_struct_or_union_definition(s, pos, typedef_flag = 0):
  # s.sy == ident 'struct' or 'union'
  kind = s.systring
  s.next()
  name = p_ident(s)
  s.type_names.append(name)
  attributes = None
  if s.sy == ':':
    s.next()
    s.expect('NEWLINE')
    s.expect_indent()
    attributes = []
    while s.sy <> 'DEDENT':
      attributes.append(
        p_c_func_or_var_declaration(s, s.position(),
          suite_allowed = 0))
    s.expect_dedent()
  else:
    s.expect('NEWLINE')
  return Nodes.CStructOrUnionDefNode(pos, 
    name = name, kind = kind, attributes = attributes,
    typedef_flag = typedef_flag)

def p_c_func_or_var_declaration(s, pos, 
    extern_flag = 0, suite_allowed = 1):
  public_flag = 0
  if s.sy == 'IDENT' and s.systring in ('public', 'private'):
    if extern_flag:
      error(s.position, "Declaration cannot be both 'extern' and '%s'"
        % s.systring)
    if s.systring == 'public':
      public_flag = 1
    s.next()
  base_type = p_c_base_type(s)
  declarator = p_c_declarator(s)
  if s.sy == ':':
    if not suite_allowed:
      s.error("Suite not allowed here")
    suite = p_suite(s, 'function')
    result = Nodes.CFuncDefNode(pos,
      extern_flag = extern_flag,
      public_flag = public_flag,
      base_type = base_type,
      declarator = declarator, 
      body = suite)
  else:
    declarators = [declarator]
    while s.sy == ',':
      s.next()
      declarator = p_c_declarator(s)
      declarators.append(declarator)
    s.expect('NEWLINE', 
      'Syntax error in C variable declaration')
    result = Nodes.CVarDefNode(pos, 
      extern_flag = extern_flag,
      public_flag = public_flag,
      base_type = base_type, 
      declarators = declarators)
  return result

def p_ctypedef_statement(s, extern_flag = 0):
  # s.sy == 'ctypedef'
  pos = s.position()
  s.next()
  if s.sy == 'IDENT' and s.systring == 'extern':
    extern_flag = 1
    s.next()
  if s.sy == 'class':
    return p_c_class_definition(s, pos,
      extern_flag = extern_flag,
      typedef_flag = 1)
  elif s.sy == 'IDENT' and \
    s.systring in ('struct', 'union', 'enum'):
      if s.systring == 'enum':
        return p_c_enum_definition(s, pos, 
          typedef_flag = 1)
      else:
        return p_c_struct_or_union_definition(s, pos,
          typedef_flag = 1)
  else:
    base_type = p_c_base_type(s)
    declarator = p_c_declarator(s, is_type = 1)
    s.expect('NEWLINE',
      "Syntax error in ctypedef statement")
    return Nodes.CTypeDefNode(pos,
      base_type = base_type, declarator = declarator)

def p_def_statement(s):
  # s.sy == 'def'
  pos = s.position()
  s.next()
  name = p_ident(s)
  args = []
  s.expect('(');
  args = p_c_arg_list(s)
  star_arg = None
  starstar_arg = None
  if s.sy == '*':
    s.next()
    star_arg = p_py_arg_decl(s)
    if s.sy == ',':
      s.next()
      if s.sy == '**':
        s.next()
        starstar_arg = p_py_arg_decl(s)
  elif s.sy == '**':
    s.next()
    starstar_arg = p_py_arg_decl(s)
  s.expect(')')
  doc, body = p_suite(s, 'function', with_doc = 1)
  return Nodes.DefNode(pos, name = name, args = args, 
    star_arg = star_arg, starstar_arg = starstar_arg,
    doc = doc, body = body)

def p_py_arg_decl(s):
  pos = s.position()
  name = p_ident(s)
  return Nodes.PyArgDeclNode(pos, name = name)

def p_class_statement(s):
  # s.sy == 'class'
  pos = s.position()
  s.next()
  class_name = p_ident(s)
  if s.sy == '(':
    s.next()
    base_list = p_simple_expr_list(s)
    s.expect(')')
  else:
    base_list = []
  doc, body = p_suite(s, 'class', with_doc = 1)
  return Nodes.PyClassDefNode(pos,
    name = class_name,
    bases = ExprNodes.TupleNode(pos, args = base_list),
    doc = doc, body = body)

def p_c_class_definition(s, pos, 
    extern_flag = 0, typedef_flag = 0):
  # s.sy == 'class'
  s.next()
  class_name = p_ident(s)
  s.type_names.append(class_name)
  objstruct_name = None
  typeobj_name = None
  if s.sy == '[':
    if not extern_flag:
      error("Only allowed for external C class", s.position())
    objstruct_name, typeobj_name = p_c_class_options(s)
  if s.sy == ':':
    doc, body = p_suite(s, 'c_class', with_doc = 1)
  else:
    s.expect('NEWLINE',
      "Syntax error in C class definition")
    doc = None
    body = None
  return Nodes.CClassDefNode(pos,
    extern_flag = extern_flag,
    typedef_flag = typedef_flag,
    name = class_name,
    objstruct_name = objstruct_name,
    typeobj_name = typeobj_name,
    doc = doc,
    body = body)

def p_c_class_options(s):
  objstruct_name = None
  typeobj_name = None
  s.expect('[')
  while 1:
    if s.sy <> 'IDENT':
      break
    if s.systring == 'object':
      s.next()
      objstruct_name = p_ident(s)
    elif s.systring == 'type':
      s.next()
      typeobj_name = p_ident(s)
    if s.sy <> ',':
      break
    s.next()
  s.expect(']', "Expected 'object' or 'type'")
  return objstruct_name, typeobj_name

def p_doc_string(s):
  if s.sy == 'STRING' or s.sy == 'BEGIN_STRING':
    result = p_cat_string_literal(s)
    s.expect('NEWLINE', "Syntax error in doc string")
    return result
  else:
    return None

def p_module(s):
  s.type_names = ["object"]
  pos = s.position()
  doc = p_doc_string(s)
  body = p_statement_list(s, 'module')
  if s.sy <> 'EOF':
    s.error("Syntax error in statement [%s,%s]" % (
      repr(s.sy), repr(s.systring)))
  return Nodes.ModuleNode(pos, doc = doc, body = body)

#----------------------------------------------
#
#   Debugging
#
#----------------------------------------------

def print_parse_tree(f, node, level, key = None):	
  ind = "  " * level
  if node:
    f.write(ind)
    if key:
      f.write("%s: " % key)
    t = type(node)
    if t == TupleType:
      f.write("(%s @ %s\n" % (node[0], node[1]))
      for i in xrange(2, len(node)):
        print_parse_tree(f, node[i], level+1)
      f.write("%s)\n" % ind)
      return
    elif isinstance(node, Node):
      try:
        tag = node.tag
      except AttributeError:
        tag = node.__class__.__name__
      f.write("%s @ %s\n" % (tag, node.pos))
      for name, value in node.__dict__.items():
        if name <> 'tag' and name <> 'pos':
          print_parse_tree(f, value, level+1, name)
      return
    elif t == ListType:
      f.write("[\n")
      for i in xrange(len(node)):
        print_parse_tree(f, node[i], level+1)
      f.write("%s]\n" % ind)
      return
  f.write("%s%s\n" % (ind, node))

