#
#   Pyrex - Symbol Table
#

from Errors import error
import Naming
from PyrexTypes import \
  py_object_type, c_char_array_type, \
  CEnumType, CStructOrUnionType
from TypeSlots import \
  pyfunction_signature, pymethod_signature, \
  get_special_method_signature

class Entry:
  # name             string     Python name of entity
  # cname            string     C name of entity
  # type             PyrexType  Type of entity
  # doc              string     Doc string
  # borrowed         bool       Is a borrowed reference
  # init             string     Initial value
  # extern_flag      boolean    Is declared extern
  # public_flag      boolean    Is declared public
  # is_builtin       boolean    Is a Python builtin name
  # is_cglobal       boolean    Is a C global variable
  # is_pyglobal      boolean    Is a Python module-level variable
  #                               or class attribute during
  #                               class construction
  # is_variable      boolean    Is a variable
  # is_cfunction     boolean    Is a C function
  # is_type          boolean    Is a type definition
  # is_const         boolean    Is a constant
  # is_self_arg      boolean    Is the "self" arg of an exttype method
  # is_readonly      boolean    Can't be assigned to
  # func_cname       string     C func implementing Python func
  # pos              position   Source position where declared
  # namespace_cname  string      if is_pyglobal, C variable
  #                              holding its home namespace
  # pymethdef_cname  string     PyMethodDef structure
  # signature        Signature  Arg & return types for Python func
  # init_to_none     boolean    True if initial value should be None
  # as_variable      Entry      Alternative interpretation of extension
  #                               type name as a variable
  # xdecref_cleanup  boolean    Use Py_XDECREF for error cleanup
  # in_cinclude      boolean    Suppress C declaration code
  # enum_values      [Entry]    For enum types, list of values
  
  borrowed = 0
  init = ""
  extern_flag = 0
  public_flag = 0
  is_builtin = 0
  is_cglobal = 0
  is_pyglobal = 0
  is_variable = 0
  is_cfunction = 0
  is_type = 0
  is_const = 0
  is_self_arg = 0
  is_readonly = 0
  doc = None
  init_to_none = 0
  as_variable = None
  xdecref_cleanup = 0
  in_cinclude = 0
  
  def __init__(self, name, cname, type, pos = None, init = None):
    self.name = name
    self.cname = cname
    self.type = type
    self.pos = pos
    self.init = init


class Scope:
  # outer_scope       Scope or None      Enclosing scope
  # entries           {string : Entry}   Python name to entry mapping
  # const_entries     [Entry]            Constant entries
  # sue_entries       [Entry]            Struct/union/enum entries
  # arg_entries       [Entry]            Function argument entries
  # var_entries       [Entry]            User-defined variable entries
  # pyfunc_entries    [Entry]            Python function entries
  # cfunc_entries     [Entry]            C function entries
  # c_class_entries   [Entry]            Extension type entries
  # temp_entries      [Entry]            Temporary variable entries
  # free_temp_entries [Entry]            Temp variables currently unused
  # temp_counter      integer            Counter for naming temp vars
  # cname_to_entry    {string : Entry}   Temp cname to entry mapping
  # pow_function_used boolean            The C pow() function is used
  # return_type       PyrexType or None  Return type of function owning scope
  # is_py_class_scope boolean            Is a Python class scope
  # is_c_class_scope  boolean            Is an extension type scope
  # scope_prefix      string             Disambiguator for C names
  # in_cinclude       boolean            Suppress C declaration code
  # function_name     string             For use in tracebacks

  is_py_class_scope = 0
  is_c_class_scope = 0
  scope_prefix = ""
  in_cinclude = 0
  
  def __init__(self, outer_scope):
    self.outer_scope = outer_scope
    self.entries = {}
    self.const_entries = []
    self.sue_entries = []
    self.arg_entries = []
    self.var_entries = []
    self.pyfunc_entries = []
    self.cfunc_entries = []
    self.c_class_entries = []
    self.temp_entries = []
    self.free_temp_entries = []
    self.pending_temp_entries = [] # TEMPORARY
    self.temp_counter = 1
    self.cname_to_entry = {}
    self.pow_function_used = 0
  
  def global_scope(self):
    # Return the module-level scope containing this scope.
    return self.outer_scope.global_scope()
  
  def declare(self, name, cname, type, pos):
    # Create new entry, and add to dictionary if
    # name is not None. Reports an error if already 
    # declared.
    d = self.entries
    if name and d.has_key(name):
      error(pos, "'%s' redeclared" % name)
    entry = Entry(name, cname, type, pos = pos)
    entry.in_cinclude = self.in_cinclude
    #print "Scope.declare:", name ###
    #print "...entry.in_cinclude =", entry.in_cinclude ###
    if name:
      d[name] = entry
    return entry
  
  def undeclare(self, name):
    del self.entries[name]
  
  def declare_const(self, name, type, value, pos):
    # Add an entry for a named constant.
    entry = self.declare(name, name, type, pos)
    entry.is_const = 1
    entry.value = value
    return entry
  
  def declare_type(self, name, type, pos, extern_flag = 0):
    # Add an entry for a type definition.
    entry = self.declare(name, name, type, pos)
    entry.extern_flag = extern_flag
    entry.is_type = 1
    return entry
    
  def declare_struct_or_union(self, name, kind, scope, 
      typedef_flag, pos):
    # Add an entry for a struct or union definition.
    entry = self.lookup_here(name)
    if not entry:
      type = CStructOrUnionType(name, kind, scope, typedef_flag)
      entry = self.declare_type(name, type, pos)
      self.sue_entries.append(entry)
    else:
      if (entry.is_type 
          and entry.type.is_struct_or_union
          and (not scope or not entry.type.scope)):
        entry.type.scope = scope
      else:
        error(pos, "'%s' already defined at this level" % name)
    return entry
  
  def declare_enum(self, name, pos, typedef_flag):
#		if name:
#			type = CEnumType(name, typedef_flag)
#			entry = self.declare_type(name, type, pos)
#		else:
#			import PyrexTypes
#			entry = Entry(None, None, PyrexTypes.c_int_type)
    if name:
      type = CEnumType(name, typedef_flag)
    else:
      import PyrexTypes
      type = PyrexTypes.c_int_type
    entry = self.declare_type(name, type, pos)
    entry.enum_values = []
    self.sue_entries.append(entry)
    return entry	
  
  def declare_var(self, name, type, pos, 
      extern_flag = 0, public_flag = 0, is_cdef = 0):
    # Add an entry for a variable.
    if extern_flag or public_flag:
      cname = name
    else:
      cname = Naming.var_prefix + name
    entry = self.declare(name, cname, type, pos)
    entry.is_variable = 1
    entry.extern_flag = extern_flag
    entry.public_flag = public_flag
    return entry
    
  def declare_builtin(self, name, pos):
    return self.outer_scope.declare_builtin(name, pos)
  
  def declare_pyfunction(self, name, pos):
    # Add an entry for a Python function.
    entry = self.declare_var(name, py_object_type, pos)
    entry.signature = pyfunction_signature
    self.pyfunc_entries.append(entry)
    return entry
  
  def register_pyfunction(self, entry):
    self.pyfunc_entries.append(entry)
  
  def declare_cfunction(self, name, type, pos, extern_flag, public_flag):
    # Add an entry for a C function.
    if extern_flag or public_flag:
      cname = name
    else:
      cname = Naming.func_prefix + name
    entry = self.declare(name, cname, type, pos)
    entry.func_cname = cname
    entry.is_cfunction = 1
    entry.extern_flag = extern_flag
    entry.public_flag = public_flag
    self.cfunc_entries.append(entry)
    return entry
  
  def declare_c_class(self, name, scope, pos, 
    objstruct_cname, typeobj_cname,
    extern_flag, typedef_flag):
    
    # This is tricky, because the name of an extension type
    # has to serve as both a type name and a module-level
    # variable name (holding the type object). So we first
    # declare it as a variable, then remove that entry
    # and re-declare it as a type, with the variable entry
    # hanging off the type entry.
    
    import PyrexTypes
    if not objstruct_cname:
      if extern_flag:
        objstruct_cname = name
      else:
        objstruct_cname = Naming.objstruct_prefix + name
    if not typeobj_cname and not extern_flag:
      typeobj_cname = Naming.typeobj_prefix + name
    prev_entry = self.lookup_here(name)
    
    # If extension type was previously forward-declared,
    # fill it in and return.
    if prev_entry and prev_entry.is_type \
      and prev_entry.type.is_extension_type:
        if prev_entry.type.objstruct_cname <> objstruct_cname:
          error(pos, "Object struct name differs from previous declaration")
        if prev_entry.type.typeobj_cname <> typeobj_cname:
          error(pos, "Type object name differs from previous declaration")
        if not prev_entry.type.scope:
          prev_entry.type.set_scope(scope)
          return prev_entry
        else:
          error(pos, "C class '%s' already defined" % name)
          # Continue so a valid entry will be returned
          prev_entry = None
    
    # Get here if no previous entry or previous
    # entry was not an extension type. Any previous
    # entry must now be a non-extension-type Python
    # variable, otherwise there is an error.
    if prev_entry:
      if not (prev_entry.is_variable
        and prev_entry.type.is_pyobject
        and not prev_entry.type.is_exension_type):
          error(pos,
            "'%s' is already declared at this level" % name)
          # Continue so a valid entry will be returned
          prev_entry = None
    
    # By now, prev_entry is None or a Python variable.
    var_entry = prev_entry
    
    # If we don't have a var entry yet, create one
    # (but only if the typeobject name is known).
    if not var_entry:
      if typeobj_cname:
        var_entry = self.declare_var(name, 
          PyrexTypes.py_object_type, pos)
    
    # If this is an external extension type with a type
    # object name, make the var entry a read-only C global 
    # var whose cname refers to the extension type's 
    # statically declared typeobject.
    if extern_flag and var_entry:
      var_entry.is_pyglobal = 0
      var_entry.is_cglobal = 1
      var_entry.cname = "((PyObject *)&%s)" % typeobj_cname
      var_entry.is_readonly = 1
      # We don't add it to the var_entries list, because
      # we don't want a declaration generated for it.
    
    # Now create the extension type entry, attaching
    # the var entry to it, if any.
    if var_entry:
      self.undeclare(name)
      #if extern_flag:
      #	var_entry.is_extern_cclass = 1
    type = PyrexTypes.py_extension_type(name, scope, 
      objstruct_cname, typeobj_cname, typedef_flag)
    entry = self.declare_type(name, type, pos, 
      extern_flag = extern_flag)
    entry.as_variable = var_entry
    self.c_class_entries.append(entry)
    return entry
  
  def find(self, name, pos):
    # Look up name, report error if not found.
    entry = self.lookup(name)
    if entry:
      return entry
    else:
      error(pos, "'%s' not defined" % name)
  
  def lookup(self, name):
    # Look up name in this scope or an enclosing one.
    # Return None if not found.
    return (self.lookup_here(name)
      or (self.outer_scope and self.outer_scope.lookup(name))
      or None)

  def lookup_here(self, name):
    # Look up in this scope only, return None if not found.
    return self.entries.get(name, None)
    
  def lookup_target(self, name):
    # Look up name in this scope only. Declare as Python
    # variable if not found.
    entry = self.lookup_here(name)
    if not entry:
      entry = self.declare_var(name, py_object_type, None)
    return entry
  
  def add_string_const(self, value):
    # Add an entry for a string constant.
    cname = self.new_const_cname()
    entry = Entry("", cname, c_char_array_type, init = value)
    self.const_entries.append(entry)
    return entry
  
  def new_const_cname(self):
    # Create a new globally-unique name for a constant.
    return self.global_scope().new_const_cname()

  def allocate_temp(self, type):
    # Allocate a temporary variable of the given type from the 
    # free list if available, otherwise create a new one.
    # Returns the cname of the variable.
    for entry in self.free_temp_entries:
      if entry.type == type:
        self.free_temp_entries.remove(entry)
        return entry.cname
    n = self.temp_counter
    self.temp_counter = n + 1
    cname = "%s%d" % (Naming.pyrex_prefix, n)
    entry = Entry("", cname, type)
    if type.is_pyobject:
      entry.init = "0"
    self.cname_to_entry[entry.cname] = entry
    self.temp_entries.append(entry)
    return entry.cname
  
  def allocate_temp_pyobject(self):
    # Allocate a temporary PyObject variable.
    return self.allocate_temp(py_object_type)
  
  def release_temp(self, cname):
    # Release a temporary variable for re-use.
    if not cname: # can happen when type of an expr is void
      return
    ## TEMPORARY HACK until flaw in algorithm
    ## for temp allocation can be redressed:
    ## Don't really release any temps until
    ## a whole expression has been processed.
    #print "Scope.release_temp:", cname ###
    self.pending_temp_entries.append(self.cname_to_entry[cname])
    return
    # Originally:
    self.free_temp_entries.append(self.cname_to_entry[cname])
  
  def recycle_pending_temps(self):
    #print "Scope.recycle_pending_temps:", \
    #	[e.cname for e in self.pending_temp_entries] ###
    self.free_temp_entries.extend(self.pending_temp_entries)
    self.pending_temp_entries = []
  
  def use_utility_code(self, new_code):
    self.global_scope().use_utility_code(new_code)
  
  def generate_library_function_declarations(self, code):
    # Generate extern decls for C library funcs used.
    if self.pow_function_used:
      code.putln("extern double pow(double, double);")
    

class LocalScope(Scope):
  
  def declare_arg(self, name, type, pos):
    # Add an entry for an argument of a function.
    entry = self.declare(name, Naming.var_prefix + name, type, pos)
    entry.is_variable = 1
    if type.is_pyobject:
      entry.init = "0"
    #entry.borrowed = 1 # Not using borrowed arg refs for now
    self.arg_entries.append(entry)
    return entry
  
  def declare_var(self, name, type, pos, 
      extern_flag = 0, public_flag = 0, is_cdef = 0):
    # Add an entry for a local variable.
    if public_flag:
      error(pos, "Local variable cannot be declared public")
    entry = Scope.declare_var(self, name, type, pos, 
      extern_flag, public_flag, is_cdef)
    entry.init_to_none = type.is_pyobject
    self.var_entries.append(entry)
    return entry
  
  def declare_global(self, name, pos):
    # Pull entry from global scope into local scope.
    if self.lookup_here(name):
      error(pos, "'%s' redeclared")
    else:
      entry = self.global_scope().lookup_target(name)
      self.entries[name] = entry
    

class ModuleScope(Scope):
  # module_name          string     Python name of the module
  # module_cname         string     C name of Python module object
  # #module_dict_cname   string     C name of module dict object
  # method_table_cname   string     C name of method table
  # doc                  string     Module doc string
  # doc_cname            string     C name of module doc string
  # const_counter        integer    Counter for naming constants
  # utility_code_used    [string]   Utility code to be included
  # default_entries      [Entry]    Function argument default entries
  # include_files        [string]           C headers to be included

  def __init__(self, name):
    Scope.__init__(self, BuiltinScope())
    self.module_name = name
    self.module_cname = Naming.module_cname
    self.module_dict_cname = Naming.moddict_cname
    self.method_table_cname = Naming.methtable_cname
    self.doc = ""
    self.doc_cname = Naming.moddoc_cname
    self.const_counter = 1
    self.utility_code_used = []
    self.default_entries = []
    self.include_files = ["Python.h"]
  
  def global_scope(self):
    return self
  
  def add_include_file(self, filename):
    if filename not in self.include_files:
      self.include_files.append(filename)
  
  def declare_var(self, name, type, pos, 
      extern_flag = 0, public_flag = 0, is_cdef = 0):
    # Add an entry for a global variable. If it is a Python
    # object type, and not declared with cdef, it will live 
    # in the module dictionary, otherwise it will be a C 
    # global variable.
    entry = Scope.declare_var(self, name, type, pos, 
      extern_flag, public_flag, is_cdef)
    if not is_cdef:
      if not (type.is_pyobject and not type.is_extension_type):
        raise InternalError(
          "Non-cdef global variable is not a generic Python object")
      entry.is_pyglobal = 1
      entry.namespace_cname = self.module_cname
    else:
      entry.is_cglobal = 1
      self.var_entries.append(entry)
    return entry
  
#	def make_cglobal(self, entry):
#		# Make a global var entry into a C global var.
#		# Can be used to convert what was previously a
#		# module dict var into a C var.
#		if not entry.is_cglobal:
#			entry.is_pyglobal = 0
#			entry.is_cglobal = 1
#			self.var_entries.append(entry)
  
  def declare_global(self, name, pos):
    pass
  
  def add_default_value(self, type):
    # Add an entry for holding a function argument
    # default value.
    cname = self.new_const_cname()
    entry = Entry("", cname, type)
    self.default_entries.append(entry)
    return entry
    
  def new_const_cname(self):
    # Create a new globally-unique name for a constant.
    n = self.const_counter
    self.const_counter = n + 1
    return "%s%d" % (Naming.const_prefix, n)
  
  def use_utility_code(self, new_code):
    #  Add string to list of utility code to be included,
    #  if not already there (tested using 'is').
    for old_code in self.utility_code_used:
      if old_code is new_code:
        return
    self.utility_code_used.append(new_code)


class BuiltinScope(Scope):
  #  The builtin namespace.
  
  def __init__(self):
    Scope.__init__(self, None)
  
  def declare_builtin(self, name, pos):
    entry = self.declare(name, name, py_object_type, pos)
    entry.is_builtin = 1
    return entry
  

class StructOrUnionScope(Scope):
  #  Namespace of a C struct or union.

  def __init__(self):
    Scope.__init__(self, None)

  def declare_var(self, name, type, pos, 
      extern_flag = 0, public_flag = 0, is_cdef = 0):
    # Add an entry for an attribute.
    entry = self.declare(name, name, type, pos)
    entry.is_variable = 1
    self.var_entries.append(entry)
    if type.is_pyobject:
      error(pos,
        "C struct/union member cannot be a Python object")
    if extern_flag:
      error(pos,
        "C struct/union member cannot be declared extern")
    return entry


class ClassScope(Scope):
  #  Abstract base class for namespace of
  #  Python class or extension type.
  #
  #  class_name     string   Pyrex name of the class
  #  scope_prefix   string   Additional prefix for names
  #                          declared in the class
  #  doc    string or None   Doc string

  def __init__(self, outer_scope, name):
    Scope.__init__(self, outer_scope)
    self.class_name = name
    self.scope_prefix = "%d%s_" % (len(name), name)
    self.doc = None

  def add_string_const(self, value):
    return self.outer_scope.add_string_const(value)


class PyClassScope(ClassScope):
  #  Namespace of a Python class.
  #
  #  class_dict_cname    string   C variable holding class dict
  #  class_obj_cname     string   C variable holding class object

  is_py_class_scope = 1
  
  def declare_var(self, name, type, pos, 
      extern_flag = 0, public_flag = 0, is_cdef = 0):
    # Add an entry for a class attribute.
    entry = Scope.declare_var(self, name, type, pos, 
      extern_flag, public_flag, is_cdef)
    entry.is_pyglobal = 1
    entry.namespace_cname = self.class_obj_cname
    return entry

  def allocate_temp(self, type):
    return self.outer_scope.allocate_temp(type)

  def release_temp(self, cname):
    self.outer_scope.release_temp(cname)

  def recycle_pending_temps(self):
    self.outer_scope.recycle_pending_temps()

  def add_default_value(self, type):
    return self.outer_scope.add_default_value(type)


class CClassScope(ClassScope):
  #  Namespace of an extension type.
  #
  #  parent_type          CClassType
  #  #typeobj_cname        string or None
  #  #objstruct_cname      string
  #  method_table_cname   string
  #  member_table_cname   string
  #  getset_table_cname   string
  #  has_pyobject_attrs   boolean  Any PyObject attributes?
  
  is_c_class_scope = 1
  method_table_cname = "<undefined>"
  member_table_cname = "<undefined>"
  getset_table_cname = "<undefined>"
  
  def __init__(self, outer_scope, name, 
      #objstruct_name, typeobj_name, 
      extern_flag):
    ClassScope.__init__(self, outer_scope, name)
    if extern_flag:
      #self.typeobj_cname = typeobj_name
      #self.objstruct_cname = objstruct_name or name
      pass
    else:
      #self.typeobj_cname = Naming.typeobj_prefix + name
      #self.objstruct_cname = Naming.objstruct_prefix + name
      self.method_table_cname = Naming.methtab_prefix + name
      self.member_table_cname = Naming.memtab_prefix + name
      self.getset_table_cname = Naming.gstab_prefix + name
    self.has_pyobject_attrs = 0

  def declare_var(self, name, type, pos, 
      extern_flag = 0, public_flag = 0, is_cdef = 0):
    # Add an entry for an attribute.
    if get_special_method_signature(name):
      error(pos, 
        "The name '%s' is reserved for a special method."
          % name)
    entry = self.declare(name, name, type, pos)
    entry.is_variable = 1
    self.var_entries.append(entry)
    if type.is_pyobject:
      self.has_pyobject_attrs = 1
    if extern_flag:
      error(pos,
        "Attribute of extension type cannot be declared extern")
    return entry

  def declare_pyfunction(self, name, pos):
    # Add an entry for a method.
    entry = self.declare(name, name, py_object_type, pos)
    special_sig = get_special_method_signature(name)
    if special_sig:
      entry.signature = special_sig
      # Special methods don't get put in the method table
    else:
      entry.signature = pymethod_signature
      self.pyfunc_entries.append(entry)
    return entry
      
  def declare_cfunction(self, name, type, pos, extern_flag):
    error(pos, "C function declaration not allowed here")
    # Declare it anyway to minimise further errors later
    return ClassScope.declare_cfunction(self,
      name, type, pos, extern_flag)
  
  def defines_any(self, names):
    # Test whether any of the given names are
    # defined in this scope.
    for name in names:
      if name in self.entries:	
        return 1
    return 0
